@@ -104,7 +104,7 @@ int sim_noemit = 0;           /* Flag for not emitting data */
 const std::string *init_method = NULL; /* method for  initialization. */
 
 static int callSolver(DATA* simData, threadData_t *threadData, string init_initMethod, string init_file,
-      double init_time, int lambda_steps, string outputVariablesAtEnd, int cpuTime, const char *argv_0);
+      double init_time, string outputVariablesAtEnd, int cpuTime, const char *argv_0);
 
 /*! \fn void setGlobalVerboseLevel(int argc, char**argv)
  *
@@ -182,6 +182,10 @@ void setGlobalVerboseLevel(int argc, char**argv)
   if(useStream[LOG_INIT])
     useStream[LOG_SOTI] = 1;
 
+  /* print LOG_STATS if LOG_SOLVER if active */
+  if(useStream[LOG_SOLVER_V] == 1)
+    useStream[LOG_SOLVER] = 1;
+
   /* print LOG_STATS if LOG_SOLVER if active */
   if(useStream[LOG_SOLVER] == 1)
     useStream[LOG_STATS] = 1;
@@ -302,14 +306,23 @@ static int getNewtonStrategy()
   return NEWTON_NONE;
 }
 
-static int getNlsLSSolver()
+static int getNlsLSSolver(int nlsSolver)
 {
   int i;
   const char *cflags = omc_flagValue[FLAG_NLS_LS];
   const string *method = cflags ? new string(cflags) : NULL;
 
   if(!method)
-    return NLS_LS_LAPACK; /* default method */
+  {
+    if (nlsSolver == NLS_KINSOL)
+    {
+      return NLS_LS_KLU; /* default kinsol linear solver method */
+    }
+    else
+    {
+      return NLS_LS_LAPACK; /* default method */
+    }
+  }
 
   for(i=1; i<NLS_LS_MAX; ++i)
     if(*method == NLS_LS_METHOD[i])
@@ -547,7 +560,6 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
   string init_time_string = "";
   double init_time = 0.0;
   string init_lambda_steps_string = "";
-  int init_lambda_steps = 1;
   string outputVariablesAtEnd = "";
   int cpuTime = omc_flag[FLAG_CPU];
 
@@ -577,16 +589,16 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
     outputVariablesAtEnd = omc_flagValue[FLAG_OUTPUT];
   }
 
-  retVal = callSolver(data, threadData, init_initMethod, init_file, init_time, init_lambda_steps, outputVariablesAtEnd, cpuTime, argv[0]);
+  retVal = callSolver(data, threadData, init_initMethod, init_file, init_time, outputVariablesAtEnd, cpuTime, argv[0]);
 
   if (omc_flag[FLAG_ALARM]) {
     alarm(0);
   }
 
   if(0 == retVal && create_linearmodel) {
-    rt_tick(SIM_TIMER_LINEARIZE);
+    rt_tick(SIM_TIMER_JACOBIAN);
     retVal = linearize(data, threadData);
-    rt_accumulate(SIM_TIMER_LINEARIZE);
+    rt_accumulate(SIM_TIMER_JACOBIAN);
     infoStreamPrint(LOG_STDOUT, 0, "Linear model is created!");
   }
 
@@ -678,7 +690,7 @@ int initializeResultData(DATA* simData, threadData_t *threadData, int cpuTime)
  * "rungekutta" calls a fourth-order Runge-Kutta Solver
  */
 static int callSolver(DATA* simData, threadData_t *threadData, string init_initMethod, string init_file,
-      double init_time, int lambda_steps, string outputVariablesAtEnd, int cpuTime, const char *argv_0)
+      double init_time, string outputVariablesAtEnd, int cpuTime, const char *argv_0)
 {
   TRACE_PUSH
   int retVal = -1;
@@ -710,7 +722,7 @@ static int callSolver(DATA* simData, threadData_t *threadData, string init_initM
   /* if no states are present, then we can
    * use euler method, since it does nothing.
    */
-  if (simData->modelData->nStates < 1 && solverID != S_OPTIMIZATION && solverID != S_SYM_EULER) {
+  if (simData->modelData->nStates < 1 && solverID != S_OPTIMIZATION && solverID != S_SYM_SOLVER) {
     solverID = S_EULER;
   }
 
@@ -732,7 +744,7 @@ static int callSolver(DATA* simData, threadData_t *threadData, string init_initM
                         simData->simulationInfo->numSteps, simData->simulationInfo->tolerance, 3);
     } else /* standard solver interface */
 #endif
-      retVal = solver_main(simData, threadData, init_initMethod.c_str(), init_file.c_str(), init_time, lambda_steps, solverID, outVars, argv_0);
+      retVal = solver_main(simData, threadData, init_initMethod.c_str(), init_file.c_str(), init_time, solverID, outVars, argv_0);
   }
 
   MMC_CATCH_INTERNAL(mmc_jumper)
@@ -831,7 +843,67 @@ int initRuntimeAndSimulation(int argc, char**argv, DATA *data, threadData_t *thr
   data->simulationInfo->lssMethod = getlinearSparseSolverMethod();
   data->simulationInfo->newtonStrategy = getNewtonStrategy();
   data->simulationInfo->nlsCsvInfomation = omc_flag[FLAG_NLS_INFO];
-  data->simulationInfo->nlsLinearSolver = getNlsLSSolver();
+  data->simulationInfo->nlsLinearSolver = getNlsLSSolver(data->simulationInfo->nlsMethod);
+
+  if(omc_flag[FLAG_HOMOTOPY_ADAPT_BEND]) {
+    homAdaptBend = atof(omc_flagValue[FLAG_HOMOTOPY_ADAPT_BEND]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homAdaptBend changed to %f", homAdaptBend);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_H_EPS]) {
+    homHEps = atof(omc_flagValue[FLAG_HOMOTOPY_H_EPS]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homHEps changed to %f", homHEps);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_MAX_LAMBDA_STEPS]) {
+    homMaxLambdaSteps = atoi(omc_flagValue[FLAG_HOMOTOPY_MAX_LAMBDA_STEPS]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homMaxLambdaSteps changed to %d", homMaxLambdaSteps);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_MAX_NEWTON_STEPS]) {
+    homMaxNewtonSteps = atoi(omc_flagValue[FLAG_HOMOTOPY_MAX_NEWTON_STEPS]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homMaxNewtonSteps changed to %d", homMaxNewtonSteps);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_MAX_TRIES]) {
+    homMaxTries = atoi(omc_flagValue[FLAG_HOMOTOPY_MAX_TRIES]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homMaxTries changed to %d", homMaxTries);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_TAU_DEC_FACTOR]) {
+    homTauDecreasingFactor = atof(omc_flagValue[FLAG_HOMOTOPY_TAU_DEC_FACTOR]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homTauDecreasingFactor changed to %f", homTauDecreasingFactor);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_TAU_DEC_FACTOR_PRED]) {
+    homTauDecreasingFactorPredictor = atof(omc_flagValue[FLAG_HOMOTOPY_TAU_DEC_FACTOR_PRED]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homTauDecreasingFactorPredictor changed to %f", homTauDecreasingFactorPredictor);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_TAU_INC_FACTOR]) {
+    homTauIncreasingFactor = atof(omc_flagValue[FLAG_HOMOTOPY_TAU_INC_FACTOR]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homTauIncreasingFactor changed to %f", homTauIncreasingFactor);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_TAU_INC_THRESHOLD]) {
+    homTauIncreasingThreshold = atof(omc_flagValue[FLAG_HOMOTOPY_TAU_INC_THRESHOLD]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homTauIncreasingThreshold changed to %f", homTauIncreasingThreshold);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_TAU_MAX]) {
+    homTauMax = atof(omc_flagValue[FLAG_HOMOTOPY_TAU_MAX]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homTauMax changed to %f", homTauMax);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_TAU_MIN]) {
+    homTauMin = atof(omc_flagValue[FLAG_HOMOTOPY_TAU_MIN]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homTauMin changed to %f", homTauMin);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_TAU_START]) {
+    homTauStart = atof(omc_flagValue[FLAG_HOMOTOPY_TAU_START]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homTauStart changed to %f", homTauStart);
+  }
 
   if(omc_flag[FLAG_LSS_MAX_DENSITY]) {
     linearSparseSolverMaxDensity = atof(omc_flagValue[FLAG_LSS_MAX_DENSITY]);
@@ -841,6 +913,14 @@ int initRuntimeAndSimulation(int argc, char**argv, DATA *data, threadData_t *thr
     linearSparseSolverMinSize = atoi(omc_flagValue[FLAG_LSS_MIN_SIZE]);
     infoStreamPrint(LOG_STDOUT, 0, "Maximum system size for using linear sparse solver changed to %d", linearSparseSolverMinSize);
   }
+  if(omc_flag[FLAG_NLS_MAX_DENSITY]) {
+    nonlinearSparseSolverMaxDensity = atof(omc_flagValue[FLAG_NLS_MAX_DENSITY]);
+    infoStreamPrint(LOG_STDOUT, 0, "Maximum density for using non-linear sparse solver changed to %f", nonlinearSparseSolverMaxDensity);
+  }
+  if(omc_flag[FLAG_NLS_MIN_SIZE]) {
+    nonlinearSparseSolverMinSize = atoi(omc_flagValue[FLAG_NLS_MIN_SIZE]);
+    infoStreamPrint(LOG_STDOUT, 0, "Maximum system size for using non-linear sparse solver changed to %d", nonlinearSparseSolverMinSize);
+  }
   if(omc_flag[FLAG_NEWTON_XTOL]) {
     newtonXTol = atof(omc_flagValue[FLAG_NEWTON_XTOL]);
     infoStreamPrint(LOG_STDOUT, 0, "Tolerance for updating solution vector in Newton solver changed to %g", newtonXTol);
@@ -851,6 +931,16 @@ int initRuntimeAndSimulation(int argc, char**argv, DATA *data, threadData_t *thr
     infoStreamPrint(LOG_STDOUT, 0, "Tolerance for accepting accuracy in Newton solver changed to %g", newtonFTol);
   }
 
+  if(omc_flag[FLAG_NEWTON_MAX_STEP_FACTOR]) {
+    maxStepFactor = atof(omc_flagValue[FLAG_NEWTON_MAX_STEP_FACTOR]);
+    infoStreamPrint(LOG_STDOUT, 0, "Maximum step size factor for a Newton step changed to %g", newtonFTol);
+  }
+
+  if(omc_flag[FLAG_STEADY_STATE_TOL]) {
+    steadyStateTol = atof(omc_flagValue[FLAG_STEADY_STATE_TOL]);
+    infoStreamPrint(LOG_STDOUT, 0, "Tolerance for steady state detection changed to %g", steadyStateTol);
+  }
+
   rt_tick(SIM_TIMER_INIT_XML);
   read_input_xml(data->modelData, data->simulationInfo);
   rt_accumulate(SIM_TIMER_INIT_XML);
@@ -1020,7 +1110,7 @@ static void messageXMLTCP(int type, int stream, int indentNext, char *msg, int s
     int i;
     xmlTcpStream << "\">\n";
     for (i=1; i<=*indexes; i++) {
-      xmlTcpStream << "<used index=\"" << indexes[i] << "%d\" />\n";
+      xmlTcpStream << "<used index=\"" << indexes[i] << "\" />\n";
     }
     if (!indentNext) {
       numOpenTags--;
