@@ -39,6 +39,7 @@
 #include "util/omc_error.h"
 #include "util/varinfo.h"
 #include "model_help.h"
+#include "simulation/options.h"
 #include "simulation/simulation_info_json.h"
 #include "util/omc_msvc.h" /* for freaking round! */
 #include "nonlinearSystem.h"
@@ -46,10 +47,35 @@
 #include "mixedSystem.h"
 #include "delay.h"
 #include "epsilon.h"
+#include "simulation/solver/stateset.h"
 #include "meta/meta_modelica.h"
 
-static const int IterationMax = 200;
+int maxEventIterations = 20;
+double linearSparseSolverMaxDensity = 0.2;
+int linearSparseSolverMinSize = 4001;
+double nonlinearSparseSolverMaxDensity = 0.2;
+int nonlinearSparseSolverMinSize = 10001;
+double maxStepFactor = 1e12;
+double newtonXTol = 1e-12;
+double newtonFTol = 1e-12;
+double steadyStateTol = 1e-3;
 const size_t SIZERINGBUFFER = 3;
+int compiledInDAEMode = 0;
+int compiledWithSymSolver = 0;
+double numericalDifferentiationDeltaXlinearize = 1e-8;
+double numericalDifferentiationDeltaXsolver = 1e-8;
+double homAdaptBend = 0.5;
+double homHEps = 1e-5;
+int homMaxLambdaSteps = 0;
+int homMaxNewtonSteps = 20;
+int homMaxTries = 10;
+double homTauDecreasingFactor = 10.0;
+double homTauDecreasingFactorPredictor = 2.0;
+double homTauIncreasingFactor = 2.0;
+double homTauIncreasingThreshold = 10.0;
+double homTauMax = 10.0;
+double homTauMin = 1e-4;
+double homTauStart = 0.2;
 
 static double tolZC;
 
@@ -63,7 +89,7 @@ static double tolZC;
 void updateDiscreteSystem(DATA *data, threadData_t *threadData)
 {
   TRACE_PUSH
-  int IterationNum = 0;
+  int numEventIterations = 0;
   int discreteChanged = 0;
   modelica_boolean relationChanged = 0;
   data->simulationInfo->needToIterate = 0;
@@ -99,9 +125,9 @@ void updateDiscreteSystem(DATA *data, threadData_t *threadData)
 
     data->callback->functionDAE(data, threadData);
 
-    IterationNum++;
-    if(IterationNum > IterationMax) {
-      throwStreamPrint(threadData, "ERROR: Too many event iterations. System is inconsistent. Simulation terminate.");
+    numEventIterations++;
+    if(numEventIterations > maxEventIterations) {
+      throwStreamPrint(threadData, "Simulation terminated due to too many, i.e. %d, event iterations.\nThis could either indicate an inconsistent system or an undersized limit of event iterations.\nThe limit of event iterations can be specified using the runtime flag '–%s=<value>'.", maxEventIterations, FLAG_NAME[FLAG_MAX_EVENT_ITERATIONS]);
     }
 
     relationChanged = checkRelations(data);
@@ -198,6 +224,7 @@ void printAllVars(DATA *data, int ringSegment, int stream)
     infoStreamPrint(stream, 0, "%ld: %s = %s (pre: %s)", i+1, mData->booleanVarsData[i].info.name, data->localData[ringSegment]->booleanVars[i] ? "true" : "false", sInfo->booleanVarsPre[i] ? "true" : "false");
   messageClose(stream);
 
+#if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
   infoStreamPrint(stream, 1, "string variables");
   for(i=0; i<mData->nVariablesString; ++i)
     infoStreamPrint(stream, 0, "%ld: %s = %s (pre: %s)", i+1,
@@ -205,7 +232,7 @@ void printAllVars(DATA *data, int ringSegment, int stream)
         MMC_STRINGDATA(data->localData[ringSegment]->stringVars[i]),
         MMC_STRINGDATA(sInfo->stringVarsPre[i]));
   messageClose(stream);
-
+#endif
   messageClose(stream);
 
   TRACE_POP
@@ -253,11 +280,12 @@ void printAllVarsDebug(DATA *data, int ringSegment, int stream)
     debugStreamPrint(stream, 0, "%ld: %s = %s (pre: %s)", i+1, mData->booleanVarsData[i].info.name, data->localData[ringSegment]->booleanVars[i] ? "true" : "false", sInfo->booleanVarsPre[i] ? "true" : "false");
   messageClose(stream);
 
+#if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
   debugStreamPrint(stream, 1, "string variables");
   for(i=0; i<mData->nVariablesString; ++i)
     debugStreamPrint(stream, 0, "%ld: %s = %s (pre: %s)", i+1, mData->stringVarsData[i].info.name, data->localData[ringSegment]->stringVars[i], sInfo->stringVarsPre[i]);
   messageClose(stream);
-
+#endif
   messageClose(stream);
 
   TRACE_POP
@@ -339,14 +367,14 @@ void printParameters(DATA *data, int stream)
  *
  *  prints sparse structure of jacobian A
  *
- *  \param [in]  [data]
+ *  \param [in]  [sparsePattern]
+ *  \param [in]  [sizeRow]
+ *  \param [in]  [sizeCol]
  *  \param [in]  [stream]
  *
- *  \author lochel
  */
-void printSparseStructure(DATA *data, int stream)
+void printSparseStructure(SPARSE_PATTERN *sparsePattern, int sizeRows, int sizeCols, int stream, const char* name)
 {
-  const int index = data->callback->INDEX_JAC_A;
   unsigned int row, col, i, j;
   /* Will crash with a static size array */
   char *buffer = NULL;
@@ -354,29 +382,19 @@ void printSparseStructure(DATA *data, int stream)
   if (!ACTIVE_STREAM(stream))
     return;
 
-  buffer = (char*)omc_alloc_interface.malloc(sizeof(char)* 2*data->simulationInfo->analyticJacobians[index].sizeCols + 4);
+  buffer = (char*)omc_alloc_interface.malloc(sizeof(char)* 2*sizeCols + 4);
 
-  infoStreamPrint(stream, 1, "sparse structure of jacobian A [size: %ux%u]", data->simulationInfo->analyticJacobians[index].sizeRows, data->simulationInfo->analyticJacobians[index].sizeCols);
-  infoStreamPrint(stream, 0, "%u nonzero elements", data->simulationInfo->analyticJacobians[index].sparsePattern.numberOfNoneZeros);
-  /*
-  sprintf(buffer, "");
-  for(row=0; row < data->simulationInfo->analyticJacobians[index].sizeRows; row++)
-    sprintf(buffer, "%s%u ", buffer, data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[row]);
-  infoStreamPrint(stream, 0, "leadindex: %s", buffer);
+  infoStreamPrint(stream, 1, "sparse structure of %s [size: %ux%u]", name, sizeRows, sizeCols);
+  infoStreamPrint(stream, 0, "%u nonzero elements", sparsePattern->numberOfNoneZeros);
 
-  sprintf(buffer, "");
-  for(i=0; i < data->simulationInfo->analyticJacobians[index].sparsePattern.numberOfNoneZeros; i++)
-    sprintf(buffer, "%s%u ", buffer, data->simulationInfo->analyticJacobians[index].sparsePattern.index[i]);
-  infoStreamPrint(stream, 0, "index: %s", buffer);
-  */
   infoStreamPrint(stream, 1, "transposed sparse structure (rows: states)");
   i=0;
-  for(row=0; row < data->simulationInfo->analyticJacobians[index].sizeRows; row++)
+  for(row=0; row < sizeRows; row++)
   {
     j=0;
-    for(col=0; i < data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[row]; col++)
+    for(col=0; i < sparsePattern->leadindex[row+1]; col++)
     {
-      if(data->simulationInfo->analyticJacobians[index].sparsePattern.index[i] == col)
+      if(sparsePattern->index[i] == col)
       {
         buffer[j++] = '*';
         ++i;
@@ -529,7 +547,9 @@ void copyRingBufferSimulationData(DATA *data, threadData_t *threadData, SIMULATI
     memcpy(destData[i]->realVars, data->localData[i]->realVars, sizeof(modelica_real)*data->modelData->nVariablesReal);
     memcpy(destData[i]->integerVars, data->localData[i]->integerVars, sizeof(modelica_integer)*data->modelData->nVariablesInteger);
     memcpy(destData[i]->booleanVars, data->localData[i]->booleanVars, sizeof(modelica_boolean)*data->modelData->nVariablesBoolean);
+#if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
     memcpy(destData[i]->stringVars, data->localData[i]->stringVars, sizeof(modelica_string)*data->modelData->nVariablesString);
+#endif
   }
 
 
@@ -559,7 +579,9 @@ void restoreExtrapolationDataOld(DATA *data)
     memcpy(data->localData[i-1]->realVars, data->localData[i]->realVars, sizeof(modelica_real)*data->modelData->nVariablesReal);
     memcpy(data->localData[i-1]->integerVars, data->localData[i]->integerVars, sizeof(modelica_integer)*data->modelData->nVariablesInteger);
     memcpy(data->localData[i-1]->booleanVars, data->localData[i]->booleanVars, sizeof(modelica_boolean)*data->modelData->nVariablesBoolean);
+#if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
     memcpy(data->localData[i-1]->stringVars, data->localData[i]->stringVars, sizeof(modelica_string)*data->modelData->nVariablesString);
+#endif
   }
 
   TRACE_POP
@@ -595,12 +617,13 @@ void setAllVarsToStart(DATA *data)
     sData->booleanVars[i] = mData->booleanVarsData[i].attribute.start;
     debugStreamPrint(LOG_DEBUG, 0, "set Boolean var %s = %s", mData->booleanVarsData[i].info.name, sData->booleanVars[i] ? "true" : "false");
   }
+#if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
   for(i=0; i<mData->nVariablesString; ++i)
   {
     sData->stringVars[i] = mmc_mk_scon_persist(mData->stringVarsData[i].attribute.start);
     debugStreamPrint(LOG_DEBUG, 0, "set String var %s = %s", mData->stringVarsData[i].info.name, MMC_STRINGDATA(sData->stringVars[i]));
   }
-
+#endif
   TRACE_POP
 }
 
@@ -635,11 +658,13 @@ void setAllStartToVars(DATA *data)
     mData->booleanVarsData[i].attribute.start = sData->booleanVars[i];
     debugStreamPrint(LOG_DEBUG, 0, "Boolean var %s(start=%s)", mData->booleanVarsData[i].info.name, sData->booleanVars[i] ? "true" : "false");
   }
+#if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
   for(i=0; i<mData->nVariablesString; ++i)
   {
     mData->stringVarsData[i].attribute.start = MMC_STRINGDATA(sData->stringVars[i]);
     debugStreamPrint(LOG_DEBUG, 0, "String var %s(start=%s)", mData->stringVarsData[i].info.name, MMC_STRINGDATA(sData->stringVars[i]));
   }
+#endif
   if (DEBUG_STREAM(LOG_DEBUG)) {
     messageClose(LOG_DEBUG);
   }
@@ -705,8 +730,9 @@ void storeOldValues(DATA *data)
   memcpy(sInfo->realVarsOld, sData->realVars, sizeof(modelica_real)*mData->nVariablesReal);
   memcpy(sInfo->integerVarsOld, sData->integerVars, sizeof(modelica_integer)*mData->nVariablesInteger);
   memcpy(sInfo->booleanVarsOld, sData->booleanVars, sizeof(modelica_boolean)*mData->nVariablesBoolean);
+#if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
   memcpy(sInfo->stringVarsOld, sData->stringVars, sizeof(modelica_string)*mData->nVariablesString);
-
+#endif
   TRACE_POP
 }
 
@@ -729,8 +755,9 @@ void restoreOldValues(DATA *data)
   memcpy(sData->realVars, sInfo->realVarsOld, sizeof(modelica_real)*mData->nVariablesReal);
   memcpy(sData->integerVars, sInfo->integerVarsOld, sizeof(modelica_integer)*mData->nVariablesInteger);
   memcpy(sData->booleanVars, sInfo->booleanVarsOld,  sizeof(modelica_boolean)*mData->nVariablesBoolean);
+#if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
   memcpy( sData->stringVars, sInfo->stringVarsOld, sizeof(modelica_string)*mData->nVariablesString);
-
+#endif
   TRACE_POP
 }
 
@@ -752,8 +779,9 @@ void storePreValues(DATA *data)
   memcpy(sInfo->realVarsPre, sData->realVars, sizeof(modelica_real)*mData->nVariablesReal);
   memcpy(sInfo->integerVarsPre, sData->integerVars, sizeof(modelica_integer)*mData->nVariablesInteger);
   memcpy(sInfo->booleanVarsPre, sData->booleanVars, sizeof(modelica_boolean)*mData->nVariablesBoolean);
+#if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
   memcpy(sInfo->stringVarsPre, sData->stringVars, sizeof(modelica_string)*mData->nVariablesString);
-
+#endif
   TRACE_POP
 }
 
@@ -869,8 +897,10 @@ void initializeDataStruc(DATA *data, threadData_t *threadData)
     assertStreamPrint(threadData, 0 != tmpSimData.integerVars, "out of memory");
     tmpSimData.booleanVars = (modelica_boolean*) calloc(data->modelData->nVariablesBoolean, sizeof(modelica_boolean));
     assertStreamPrint(threadData, 0 != tmpSimData.booleanVars, "out of memory");
+#if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
     tmpSimData.stringVars = (modelica_string*) omc_alloc_interface.malloc_uncollectable(data->modelData->nVariablesString * sizeof(modelica_string));
     assertStreamPrint(threadData, 0 != tmpSimData.stringVars, "out of memory");
+#endif
     appendRingData(data->simulationData, &tmpSimData);
   }
   data->localData = (SIMULATION_DATA**) omc_alloc_interface.malloc_uncollectable(SIZERINGBUFFER * sizeof(SIMULATION_DATA));
@@ -881,8 +911,9 @@ void initializeDataStruc(DATA *data, threadData_t *threadData)
   data->modelData->realVarsData = (STATIC_REAL_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nVariablesReal * sizeof(STATIC_REAL_DATA));
   data->modelData->integerVarsData = (STATIC_INTEGER_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nVariablesInteger * sizeof(STATIC_INTEGER_DATA));
   data->modelData->booleanVarsData = (STATIC_BOOLEAN_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nVariablesBoolean * sizeof(STATIC_BOOLEAN_DATA));
+#if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
   data->modelData->stringVarsData = (STATIC_STRING_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nVariablesString * sizeof(STATIC_STRING_DATA));
-
+#endif
   data->modelData->realParameterData = (STATIC_REAL_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nParametersReal * sizeof(STATIC_REAL_DATA));
   data->modelData->integerParameterData = (STATIC_INTEGER_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nParametersInteger * sizeof(STATIC_INTEGER_DATA));
   data->modelData->booleanParameterData = (STATIC_BOOLEAN_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nParametersBoolean * sizeof(STATIC_BOOLEAN_DATA));
@@ -909,6 +940,7 @@ void initializeDataStruc(DATA *data, threadData_t *threadData)
   data->simulationInfo->nlsMethod = NLS_HOMOTOPY;
 #endif
   data->simulationInfo->lsMethod = LS_LAPACK;
+  data->simulationInfo->lssMethod = LS_UMFPACK;
   data->simulationInfo->mixedMethod = MIXED_SEARCH;
   data->simulationInfo->newtonStrategy = NEWTON_PURE;
   data->simulationInfo->nlsCsvInfomation = 0;
@@ -931,14 +963,16 @@ void initializeDataStruc(DATA *data, threadData_t *threadData)
   data->simulationInfo->realVarsOld = (modelica_real*) calloc(data->modelData->nVariablesReal, sizeof(modelica_real));
   data->simulationInfo->integerVarsOld = (modelica_integer*) calloc(data->modelData->nVariablesInteger, sizeof(modelica_integer));
   data->simulationInfo->booleanVarsOld = (modelica_boolean*) calloc(data->modelData->nVariablesBoolean, sizeof(modelica_boolean));
+#if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
   data->simulationInfo->stringVarsOld = (modelica_string*) omc_alloc_interface.malloc_uncollectable(data->modelData->nVariablesString * sizeof(modelica_string));
-
+#endif
   /* buffer for all variable pre values */
   data->simulationInfo->realVarsPre = (modelica_real*) calloc(data->modelData->nVariablesReal, sizeof(modelica_real));
   data->simulationInfo->integerVarsPre = (modelica_integer*) calloc(data->modelData->nVariablesInteger, sizeof(modelica_integer));
   data->simulationInfo->booleanVarsPre = (modelica_boolean*) calloc(data->modelData->nVariablesBoolean, sizeof(modelica_boolean));
+#if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
   data->simulationInfo->stringVarsPre = (modelica_string*) omc_alloc_interface.malloc_uncollectable(data->modelData->nVariablesString * sizeof(modelica_string));
-
+#endif
   /* buffer for all parameters values */
   data->simulationInfo->realParameter = (modelica_real*) calloc(data->modelData->nParametersReal, sizeof(modelica_real));
   data->simulationInfo->integerParameter = (modelica_integer*) calloc(data->modelData->nParametersInteger, sizeof(modelica_integer));
@@ -948,21 +982,55 @@ void initializeDataStruc(DATA *data, threadData_t *threadData)
   data->simulationInfo->inputVars = (modelica_real*) calloc(data->modelData->nInputVars, sizeof(modelica_real));
   data->simulationInfo->outputVars = (modelica_real*) calloc(data->modelData->nOutputVars, sizeof(modelica_real));
 
+#if !defined(OMC_NUM_MIXED_SYSTEMS) || OMC_NUM_MIXED_SYSTEMS>0
   /* buffer for mixed systems */
-  data->simulationInfo->mixedSystemData = (MIXED_SYSTEM_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nMixedSystems*sizeof(MIXED_SYSTEM_DATA));
-  data->callback->initialMixedSystem(data->modelData->nMixedSystems, data->simulationInfo->mixedSystemData);
+#if !defined(OMC_NUM_MIXED_SYSTEMS)
+  if (data->modelData->nMixedSystems)
+#endif
+  {
+    data->simulationInfo->mixedSystemData = (MIXED_SYSTEM_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nMixedSystems*sizeof(MIXED_SYSTEM_DATA));
+    data->callback->initialMixedSystem(data->modelData->nMixedSystems, data->simulationInfo->mixedSystemData);
+  }
+#endif
 
+#if !defined(OMC_NUM_LINEAR_SYSTEMS) || OMC_NUM_LINEAR_SYSTEMS>0
   /* buffer for linear systems */
-  data->simulationInfo->linearSystemData = (LINEAR_SYSTEM_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nLinearSystems*sizeof(LINEAR_SYSTEM_DATA));
-  data->callback->initialLinearSystem(data->modelData->nLinearSystems, data->simulationInfo->linearSystemData);
+#if !defined(OMC_NUM_LINEAR_SYSTEMS)
+  if (data->modelData->nLinearSystems)
+#endif
+  {
+    data->simulationInfo->linearSystemData = (LINEAR_SYSTEM_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nLinearSystems*sizeof(LINEAR_SYSTEM_DATA));
+    data->callback->initialLinearSystem(data->modelData->nLinearSystems, data->simulationInfo->linearSystemData);
+  }
+#endif
 
+#if !defined(OMC_NUM_NONLINEAR_SYSTEMS) || OMC_NUM_NONLINEAR_SYSTEMS>0
   /* buffer for non-linear systems */
-  data->simulationInfo->nonlinearSystemData = (NONLINEAR_SYSTEM_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nNonLinearSystems*sizeof(NONLINEAR_SYSTEM_DATA));
-  data->callback->initialNonLinearSystem(data->modelData->nNonLinearSystems, data->simulationInfo->nonlinearSystemData);
+#if !defined(OMC_NUM_NONLINEAR_SYSTEMS)
+  if (data->modelData->nNonLinearSystems)
+#endif
+  {
+    data->simulationInfo->nonlinearSystemData = (NONLINEAR_SYSTEM_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nNonLinearSystems*sizeof(NONLINEAR_SYSTEM_DATA));
+    data->callback->initialNonLinearSystem(data->modelData->nNonLinearSystems, data->simulationInfo->nonlinearSystemData);
+  }
+#endif
 
+#if !defined(OMC_NO_STATESELECTION)
   /* buffer for state sets */
-  data->simulationInfo->stateSetData = (STATE_SET_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nStateSets*sizeof(STATE_SET_DATA));
-  data->callback->initializeStateSets(data->modelData->nStateSets, data->simulationInfo->stateSetData, data);
+  if (data->modelData->nStateSets) {
+    data->simulationInfo->stateSetData = (STATE_SET_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nStateSets*sizeof(STATE_SET_DATA));
+    data->callback->initializeStateSets(data->modelData->nStateSets, data->simulationInfo->stateSetData, data);
+  }
+#endif
+
+  /* buffer for daeMode */
+  data->simulationInfo->daeModeData = (DAEMODE_DATA*) omc_alloc_interface.malloc_uncollectable(sizeof(DAEMODE_DATA));
+  data->callback->initializeDAEmodeData(data, data->simulationInfo->daeModeData);
+
+  /* buffer for inline Data */
+  data->simulationInfo->inlineData = (INLINE_DATA*) omc_alloc_interface.malloc_uncollectable(sizeof(INLINE_DATA));
+  data->simulationInfo->inlineData->algVars = (modelica_real*) calloc(data->modelData->nStates, sizeof(modelica_real));
+  data->simulationInfo->inlineData->algOldVars = (modelica_real*) calloc(data->modelData->nStates, sizeof(modelica_real));
 
   /* buffer for analytical jacobians */
   data->simulationInfo->analyticJacobians = (ANALYTIC_JACOBIAN*) omc_alloc_interface.malloc_uncollectable(data->modelData->nJacobians*sizeof(ANALYTIC_JACOBIAN));
@@ -976,6 +1044,7 @@ void initializeDataStruc(DATA *data, threadData_t *threadData)
 
   assertStreamPrint(threadData, 0 != data->simulationInfo->extObjs, "error allocating external objects");
 
+#if !defined(OMC_MINIMAL_LOGGING)
   /* initial chattering info */
   data->simulationInfo->chatteringInfo.numEventLimit = 100;
   data->simulationInfo->chatteringInfo.lastSteps = (int*) calloc(data->simulationInfo->chatteringInfo.numEventLimit, sizeof(int));
@@ -983,12 +1052,15 @@ void initializeDataStruc(DATA *data, threadData_t *threadData)
   data->simulationInfo->chatteringInfo.currentIndex = 0;
   data->simulationInfo->chatteringInfo.lastStepsNumStateEvents = 0;
   data->simulationInfo->chatteringInfo.messageEmitted = 0;
+#endif
 
   /* initial call statistics */
   data->simulationInfo->callStatistics.functionODE = 0;
+  data->simulationInfo->callStatistics.functionEvalDAE = 0;
   data->simulationInfo->callStatistics.updateDiscreteSystem = 0;
   data->simulationInfo->callStatistics.functionZeroCrossingsEquations = 0;
   data->simulationInfo->callStatistics.functionZeroCrossings = 0;
+  data->simulationInfo->callStatistics.functionAlgebraics = 0;
 
   data->simulationInfo->lambda = 1.0;
 
@@ -1006,11 +1078,27 @@ void initializeDataStruc(DATA *data, threadData_t *threadData)
   data->simulationInfo->simulationSuccess = 0;
 
   /* initial delay */
+#if !defined(OMC_NDELAY_EXPRESSIONS) || OMC_NDELAY_EXPRESSIONS>0
   data->simulationInfo->delayStructure = (RINGBUFFER**)malloc(data->modelData->nDelayExpressions * sizeof(RINGBUFFER*));
   assertStreamPrint(threadData, 0 != data->simulationInfo->delayStructure, "out of memory");
 
   for(i=0; i<data->modelData->nDelayExpressions; i++)
     data->simulationInfo->delayStructure[i] = allocRingBuffer(1024, sizeof(TIME_AND_VALUE));
+#endif
+
+#if !defined(OMC_NO_STATESELECTION)
+  /* allocate memory for state selection */
+  initializeStateSetJacobians(data, threadData);
+#endif
+
+  /* allocate memory for sensitivity analysis */
+  if (omc_flag[FLAG_IDAS])
+  {
+    data->simulationInfo->sensitivityParList = (int*) calloc(data->modelData->nSensitivityParamVars, sizeof(int));
+    data->simulationInfo->sensitivityMatrix = (modelica_real*) calloc(data->modelData->nSensitivityVars-data->modelData->nSensitivityParamVars, sizeof(modelica_real));
+    data->modelData->realSensitivityData = (STATIC_REAL_DATA*) omc_alloc_interface.malloc_uncollectable(data->modelData->nSensitivityVars * sizeof(STATIC_REAL_DATA));
+  }
+
 
   TRACE_POP
 }
@@ -1052,7 +1140,9 @@ void deInitializeDataStruc(DATA *data)
   FREE_VARS(nVariablesReal,realVarsData)
   FREE_VARS(nVariablesInteger,integerVarsData)
   FREE_VARS(nVariablesBoolean,booleanVarsData)
+#if !defined(OMC_NVAR_STRING) || OMC_NVAR_STRING>0
   FREE_VARS(nVariablesString,stringVarsData)
+#endif
   FREE_VARS(nParametersReal,realParameterData)
   FREE_VARS(nParametersInteger,integerParameterData)
   FREE_VARS(nParametersBoolean,booleanParameterData)
@@ -1096,21 +1186,33 @@ void deInitializeDataStruc(DATA *data)
   free(data->simulationInfo->booleanParameter);
   omc_alloc_interface.free_uncollectable(data->simulationInfo->stringParameter);
 
-  /* free buffer for state sets */
-  omc_alloc_interface.free_uncollectable(data->simulationInfo->stateSetData);
+  if (data->modelData->nStateSets) {
+    /* free buffer for state sets */
+    omc_alloc_interface.free_uncollectable(data->simulationInfo->stateSetData);
+  }
 
-  /* free buffer of mixed systems */
-  omc_alloc_interface.free_uncollectable(data->simulationInfo->mixedSystemData);
+  if (data->modelData->nMixedSystems) {
+    /* free buffer of mixed systems */
+    omc_alloc_interface.free_uncollectable(data->simulationInfo->mixedSystemData);
+  }
 
-  /* free buffer of linear systems */
-  omc_alloc_interface.free_uncollectable(data->simulationInfo->linearSystemData);
+  if (data->modelData->nLinearSystems) {
+    /* free buffer of linear systems */
+    omc_alloc_interface.free_uncollectable(data->simulationInfo->linearSystemData);
+  }
 
-  /* free buffer of non-linear systems */
-  omc_alloc_interface.free_uncollectable(data->simulationInfo->nonlinearSystemData);
+  if (data->modelData->nNonLinearSystems)
+  {
+    /* free buffer of non-linear systems */
+    omc_alloc_interface.free_uncollectable(data->simulationInfo->nonlinearSystemData);
+  }
 
   /* free buffer jacobians */
   omc_alloc_interface.free_uncollectable(data->simulationInfo->analyticJacobians);
 
+  /* free buffer for state sets */
+  omc_alloc_interface.free_uncollectable(data->simulationInfo->daeModeData);
+
   /* free inputs and output */
   free(data->simulationInfo->inputVars);
   free(data->simulationInfo->outputVars);
@@ -1128,6 +1230,19 @@ void deInitializeDataStruc(DATA *data)
 
   free(data->simulationInfo->delayStructure);
 
+#if !defined(OMC_NO_STATESELECTION)
+  /* free stateset data */
+  freeStateSetData(data);
+#endif
+
+  /* free parameter sensitivities */
+  if (omc_flag[FLAG_IDAS])
+  {
+    free(data->simulationInfo->sensitivityParList);
+    free(data->simulationInfo->sensitivityMatrix);
+    FREE_VARS(nSensitivityVars, realSensitivityData)
+  }
+
   TRACE_POP
 }
 
@@ -1203,15 +1318,13 @@ modelica_boolean GreaterEq(double a, double b)
 modelica_integer _event_integer(modelica_real x, modelica_integer index, DATA *data)
 {
   modelica_real value;
-  if(data->simulationInfo->discreteCall == 0 || data->simulationInfo->solveContinuous)
-  {
-    value = data->simulationInfo->mathEventsValuePre[index];
-  }
-  else
+  if(data->simulationInfo->discreteCall && !data->simulationInfo->solveContinuous)
   {
     data->simulationInfo->mathEventsValuePre[index] = (modelica_integer)floor(x);
-    value = data->simulationInfo->mathEventsValuePre[index];
   }
+
+  value = data->simulationInfo->mathEventsValuePre[index];
+
   return value;
 }
 
@@ -1227,13 +1340,13 @@ modelica_integer _event_integer(modelica_real x, modelica_integer index, DATA *d
 modelica_real _event_floor(modelica_real x, modelica_integer index, DATA *data)
 {
   modelica_real value;
-  if(data->simulationInfo->discreteCall == 0 || data->simulationInfo->solveContinuous)
-    value = data->simulationInfo->mathEventsValuePre[index];
-  else
+  if(data->simulationInfo->discreteCall && !data->simulationInfo->solveContinuous)
   {
     data->simulationInfo->mathEventsValuePre[index] = x;
-    value = data->simulationInfo->mathEventsValuePre[index];
   }
+
+  value = data->simulationInfo->mathEventsValuePre[index];
+
   return (modelica_real)floor(value);
 }
 
@@ -1249,16 +1362,52 @@ modelica_real _event_floor(modelica_real x, modelica_integer index, DATA *data)
 modelica_real _event_ceil(modelica_real x, modelica_integer index, DATA *data)
 {
   modelica_real value;
-  if(data->simulationInfo->discreteCall == 0 || data->simulationInfo->solveContinuous)
-    value = data->simulationInfo->mathEventsValuePre[index];
-  else
+  if(data->simulationInfo->discreteCall && !data->simulationInfo->solveContinuous)
   {
     data->simulationInfo->mathEventsValuePre[index] = x;
-    value = data->simulationInfo->mathEventsValuePre[index];
   }
+
+  value = data->simulationInfo->mathEventsValuePre[index];
+
   return (modelica_real)ceil(value);
 }
 
+/*! \fn _event_mod_integer
+ *
+ *  \param [in]  [x1]
+ *  \param [in]  [x2]
+ *  \param [in]  [index]
+ *  \param [ref] [data]
+ */
+modelica_integer _event_mod_integer(modelica_integer x1, modelica_integer x2, modelica_integer index, DATA *data, threadData_t *threadData)
+{
+  if(data->simulationInfo->discreteCall && !data->simulationInfo->solveContinuous)
+  {
+    data->simulationInfo->mathEventsValuePre[index] = (modelica_real)x1;
+    data->simulationInfo->mathEventsValuePre[index+1] = (modelica_real)x2;
+  }
+
+  return x1 - (x1 / x2) * x2;
+}
+
+/*! \fn _event_mod_real
+ *
+ *  \param [in]  [x1]
+ *  \param [in]  [x2]
+ *  \param [in]  [index]
+ *  \param [ref] [data]
+ */
+modelica_real _event_mod_real(modelica_real x1, modelica_real x2, modelica_integer index, DATA *data, threadData_t *threadData)
+{
+  if(data->simulationInfo->discreteCall && !data->simulationInfo->solveContinuous)
+  {
+    data->simulationInfo->mathEventsValuePre[index] = x1;
+    data->simulationInfo->mathEventsValuePre[index+1] = x2;
+  }
+
+  return x1 - floor(x1 / x2) * x2;
+}
+
 /*! \fn _event_div_integer
  *
  *  \param [in]  [x1]
@@ -1271,18 +1420,15 @@ modelica_real _event_ceil(modelica_real x, modelica_integer index, DATA *data)
 modelica_integer _event_div_integer(modelica_integer x1, modelica_integer x2, modelica_integer index, DATA *data, threadData_t *threadData)
 {
   modelica_integer value1, value2;
-  if(data->simulationInfo->discreteCall == 0 || data->simulationInfo->solveContinuous)
-  {
-    value1 = (modelica_integer)data->simulationInfo->mathEventsValuePre[index];
-    value2 = (modelica_integer)data->simulationInfo->mathEventsValuePre[index+1];
-  }
-  else
+  if(data->simulationInfo->discreteCall && !data->simulationInfo->solveContinuous)
   {
     data->simulationInfo->mathEventsValuePre[index] = (modelica_real)x1;
     data->simulationInfo->mathEventsValuePre[index+1] = (modelica_real)x2;
-    value1 = (modelica_integer)data->simulationInfo->mathEventsValuePre[index];
-    value2 = (modelica_integer)data->simulationInfo->mathEventsValuePre[index+1];
   }
+
+  value1 = (modelica_integer)data->simulationInfo->mathEventsValuePre[index];
+  value2 = (modelica_integer)data->simulationInfo->mathEventsValuePre[index+1];
+
   assertStreamPrint(threadData, value2 != 0, "event_div_integer failt at time %f because x2 is zero!", data->localData[0]->timeValue);
   return ldiv(value1, value2).quot;
 }
@@ -1299,18 +1445,15 @@ modelica_integer _event_div_integer(modelica_integer x1, modelica_integer x2, mo
 modelica_real _event_div_real(modelica_real x1, modelica_real x2, modelica_integer index, DATA *data, threadData_t *threadData)
 {
   modelica_real value1, value2;
-  if(data->simulationInfo->discreteCall == 0 || data->simulationInfo->solveContinuous)
-  {
-    value1 = data->simulationInfo->mathEventsValuePre[index];
-    value2 = data->simulationInfo->mathEventsValuePre[index+1];
-  }
-  else
+  if(data->simulationInfo->discreteCall && !data->simulationInfo->solveContinuous)
   {
     data->simulationInfo->mathEventsValuePre[index] = x1;
     data->simulationInfo->mathEventsValuePre[index+1] = x2;
-    value1 = data->simulationInfo->mathEventsValuePre[index];
-    value2 = data->simulationInfo->mathEventsValuePre[index+1];
   }
+
+  value1 = data->simulationInfo->mathEventsValuePre[index];
+  value2 = data->simulationInfo->mathEventsValuePre[index+1];
+
 #if defined(_MSC_VER)
   {
     modelica_real rtmp = value1/value2;
@@ -1343,7 +1486,7 @@ const char *context_string[CONTEXT_MAX] = {
 void setContext(DATA* data, double* currentTime, int currentContext){
   data->simulationInfo->currentContextOld =  data->simulationInfo->currentContext;
   data->simulationInfo->currentContext =  currentContext;
-  infoStreamPrint(LOG_SOLVER, 0, "+++ Set context %s +++ at time %f", context_string[data->simulationInfo->currentContext], *currentTime);
+  infoStreamPrint(LOG_SOLVER_CONTEXT, 0, "+++ Set context %s +++ at time %f", context_string[data->simulationInfo->currentContext], *currentTime);
   if (currentContext == CONTEXT_JACOBIAN){
     data->simulationInfo->currentJacobianEval = 0;
   }
@@ -1358,7 +1501,7 @@ void setContext(DATA* data, double* currentTime, int currentContext){
 void increaseJacContext(DATA* data){
   if (data->simulationInfo->currentContext == CONTEXT_JACOBIAN){
     data->simulationInfo->currentJacobianEval++;
-    infoStreamPrint(LOG_SOLVER, 0, "+++ Increase Jacobian column context %s +++ to %d", context_string[data->simulationInfo->currentContext], data->simulationInfo->currentJacobianEval);
+    infoStreamPrint(LOG_SOLVER_CONTEXT, 0, "+++ Increase Jacobian column context %s +++ to %d", context_string[data->simulationInfo->currentContext], data->simulationInfo->currentJacobianEval);
   }
 }
 
@@ -1369,6 +1512,6 @@ void increaseJacContext(DATA* data){
  * Restores previous context in simulation info object
  */
 void unsetContext(DATA* data){
-  infoStreamPrint(LOG_SOLVER, 0, "--- Unset context %s ---", context_string[data->simulationInfo->currentContext]);
+  infoStreamPrint(LOG_SOLVER_CONTEXT, 0, "--- Unset context %s ---", context_string[data->simulationInfo->currentContext]);
   data->simulationInfo->currentContext =  data->simulationInfo->currentContextOld;
 }
