@@ -74,7 +74,8 @@ Pipecylinder::Pipecylinder(float rI, float rO, float l) :
   //VERTICES
   osg::Vec3Array* vertices = new osg::Vec3Array;
   osg::Vec3Array* normals = new osg::Vec3Array;
-
+  osg::Vec2Array* texcoords = new osg::Vec2Array;
+  float radiusRatioShift = rI/rO*0.5f;
   //we need so set the vertices multiple times to have a unique set of vertices per facet.
   //Thats needed for the normals which are assigned per vertex.
   //Also keep in mind to normalize the normals
@@ -84,12 +85,14 @@ Pipecylinder::Pipecylinder(float rI, float rO, float l) :
   for (int i = 0; i < nEdges; i++)  {
     vertices->push_back(osg::Vec3(sin(phi*i)*rI, cos(phi*i)*rI, 0));
     normals->push_back(osg::Vec3(0.0f,0.0f,-1.0f));
+    texcoords->push_back(osg::Vec2((sin(phi*i)*radiusRatioShift)+0.5f, (cos(phi*i)*radiusRatioShift)+0.5f));
     vertIdx++;
   }
   // outer base ring
   for (int i = 0; i < nEdges; i++)  {
     vertices->push_back(osg::Vec3(sin(phi*i)*rO, cos(phi*i)*rO, 0));
     normals->push_back(osg::Vec3(0.0f,0.0f,-1.0f));
+    texcoords->push_back((osg::Vec2((sin(phi*i)*0.5f)+0.5f, (cos(phi*i)*0.5f)+0.5f)));
     vertIdx++;
   }
   //THE TOP PLANE VERTICES
@@ -97,12 +100,14 @@ Pipecylinder::Pipecylinder(float rI, float rO, float l) :
   for (int i = 0; i < nEdges; i++) {
     vertices->push_back(osg::Vec3(sin(phi*i)*rI, cos(phi*i)*rI, l));
     normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));
+    texcoords->push_back(osg::Vec2((sin(phi*i)*radiusRatioShift)+0.5f, (cos(phi*i)*radiusRatioShift)+0.5f));
     vertIdx++;
   }
   // outer end ring
   for (int i = 0; i < nEdges; i++) {
     vertices->push_back(osg::Vec3(sin(phi*i)*rO, cos(phi*i)*rO, l));
     normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));
+    texcoords->push_back(osg::Vec2((sin(phi*i)*0.5f)+0.5f, (cos(phi*i)*0.5f)+0.5f));
     vertIdx++;
   }
 
@@ -155,6 +160,11 @@ Pipecylinder::Pipecylinder(float rI, float rO, float l) :
     normals->push_back(normalizeVec(osg::Vec3(sin(phiN)*rO, cos(phiN)*rO, 0)));
     normals->push_back(normalizeVec(osg::Vec3(sin(phiN)*rO, cos(phiN)*rO, 0)));
     normals->push_back(normalizeVec(osg::Vec3(sin(phiN)*rO, cos(phiN)*rO, 0)));
+    //the texture coordinates
+    texcoords->push_back(osg::Vec2(1.0f*i/(nEdges), 1.0f));
+    texcoords->push_back(osg::Vec2(1.0f*j/(nEdges), 1.0f));
+    texcoords->push_back(osg::Vec2(1.0f*j/(nEdges), 0.0f));
+    texcoords->push_back(osg::Vec2(1.0f*i/(nEdges), 0.0f));
     //the planes
     basePlane = new osg::DrawElementsUInt(osg::PrimitiveSet::QUADS, 0);
     basePlane->push_back(vertIdx);
@@ -179,6 +189,11 @@ Pipecylinder::Pipecylinder(float rI, float rO, float l) :
     normals->push_back(normalizeVec(osg::Vec3(-sin(phiN)*rI, -cos(phiN)*rI, 0)));
     normals->push_back(normalizeVec(osg::Vec3(-sin(phiN)*rI, -cos(phiN)*rI, 0)));
     normals->push_back(normalizeVec(osg::Vec3(-sin(phiN)*rI, -cos(phiN)*rI, 0)));
+    //the texture coordinates
+    texcoords->push_back(osg::Vec2(1.0f*i/(nEdges), 0.0f));
+    texcoords->push_back(osg::Vec2(1.0f*j/(nEdges), 0.0f));
+    texcoords->push_back(osg::Vec2(1.0f*j/(nEdges), 1.0f));
+    texcoords->push_back(osg::Vec2(1.0f*i/(nEdges), 1.0f));
     //the planes
     basePlane = new osg::DrawElementsUInt(osg::PrimitiveSet::QUADS, 0);
     basePlane->push_back(vertIdx);
@@ -191,6 +206,7 @@ Pipecylinder::Pipecylinder(float rI, float rO, float l) :
 
   this->setVertexArray(vertices);
   this->setNormalArray(normals);
+  this->setTexCoordArray(0,texcoords);
   this->setNormalBinding( osg::Geometry::BIND_PER_VERTEX);
 }
 
@@ -682,9 +698,6 @@ DXFile::DXFile(std::string filename)
         (*facette)[1] = (i * 4) + 1;
         (*facette)[2] = (i * 4) + 2;
         this->addPrimitiveSet(facette);
-
-        //normal calculation
-        osg::Vec3f normal =  faces[i].calcNormals();
       }
       else
       {
