@@ -29,17 +29,18 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #ifndef LIBRARYTREEWIDGET_H
 #define LIBRARYTREEWIDGET_H
 
-#include "MainWindow.h"
-#include "StringHandler.h"
+#include "OMC/OMCProxy.h"
+#include "Util/StringHandler.h"
+
+#include <QItemDelegate>
+#include <QTreeView>
+#include <QSortFilterProxyModel>
 
 class ItemDelegate : public QItemDelegate
 {
@@ -60,6 +61,10 @@ public:
   void drawHover(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
   virtual QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const;
   virtual bool editorEvent(QEvent *event, QAbstractItemModel *model, const QStyleOptionViewItem &option, const QModelIndex &index);
+  virtual QWidget* createEditor(QWidget *pParent, const QStyleOptionViewItem &option, const QModelIndex &index) const;
+  virtual void setEditorData(QWidget *editor, const QModelIndex &index) const;
+public slots:
+  void unitComboBoxChanged(QString text);
 };
 
 class ModelWidget;
@@ -72,9 +77,9 @@ class LibraryTreeItem : public QObject
   Q_OBJECT
 public:
   enum LibraryType {
-    Modelica,   /* Used to represent Modelica models. */
-    Text,       /* Used to represent text based files. */
-    TLM         /* Used to represent TLM files. */
+    Modelica,    /* Used to represent Modelica models. */
+    Text,        /* Used to represent text based files. */
+    CompositeModel    /* Used to represent CompositeModel files. */
   };
   enum SaveContentsType {
     SaveInOneFile,
@@ -85,7 +90,9 @@ public:
                   QString fileName, bool isSaved, LibraryTreeItem *pParent = 0);
   ~LibraryTreeItem();
   bool isRootItem() {return mIsRootItem;}
-  QList<LibraryTreeItem*> getChildren() const {return mChildren;}
+  int childrenSize() const {return mChildren.size();}
+  LibraryTreeItem* childAt(int index) const {return mChildren.at(index);}
+  QList<LibraryTreeItem*> childrenItems() {return mChildren;}
   LibraryType getLibraryType() {return mLibraryType;}
   void setLibraryType(LibraryType libraryType) {mLibraryType = libraryType;}
   void setSystemLibrary(bool systemLibrary) {mSystemLibrary = systemLibrary;}
@@ -98,21 +105,20 @@ public:
   const QString& getNameStructure() {return mNameStructure;}
   void setClassInformation(OMCInterface::getClassInformation_res classInformation);
   void setFileName(QString fileName) {mFileName = fileName;}
-  const QString& getFileName() {return mFileName;}
+  const QString& getFileName() const {return mFileName;}
   bool isFilePathValid();
   void setReadOnly(bool readOnly) {mReadOnly = readOnly;}
   bool isReadOnly() {return mReadOnly;}
   void setIsSaved(bool isSaved) {mIsSaved = isSaved;}
   bool isSaved() {return mIsSaved;}
-  bool isProtected() {return mClassInformation.isProtectedClass;}
+  bool isProtected() {return mLibraryType == LibraryTreeItem::Modelica ? mClassInformation.isProtectedClass : false;}
   bool isDocumentationClass();
-  StringHandler::ModelicaClasses getRestriction() {return StringHandler::getModelicaClassType(mClassInformation.restriction);}
+  StringHandler::ModelicaClasses getRestriction() const {return StringHandler::getModelicaClassType(mClassInformation.restriction);}
   bool isConnector() {return (getRestriction() == StringHandler::ExpandableConnector || getRestriction() == StringHandler::Connector);}
   bool isPartial() {return mClassInformation.partialPrefix;}
+  bool isState() {return mClassInformation.state;}
   void setSaveContentsType(LibraryTreeItem::SaveContentsType saveContentsType) {mSaveContentsType = saveContentsType;}
   SaveContentsType getSaveContentsType() {return mSaveContentsType;}
-  void setToolTip(QString toolTip) {mToolTip = toolTip;}
-  void setIcon(QIcon icon) {mIcon = icon;}
   void setPixmap(QPixmap pixmap) {mPixmap = pixmap;}
   QPixmap getPixmap() {return mPixmap;}
   void setDragPixmap(QPixmap dragPixmap) {mDragPixmap = dragPixmap;}
@@ -127,9 +133,9 @@ public:
   bool isExpanded() const {return mExpanded;}
   void setNonExisting(bool nonExisting) {mNonExisting = nonExisting;}
   bool isNonExisting() const {return mNonExisting;}
-  void updateAttributes();
-  QIcon getLibraryTreeItemIcon();
-  bool inRange(int lineNumber) {return (lineNumber >= mClassInformation.lineNumberStart) && (lineNumber <= mClassInformation.lineNumberEnd);}
+  QString getTooltip() const;
+  QIcon getLibraryTreeItemIcon() const;
+  bool inRange(int lineNumber);
   bool isInPackageOneFile();
   void insertChild(int position, LibraryTreeItem *pLibraryTreeItem);
   LibraryTreeItem* child(int row);
@@ -149,6 +155,7 @@ public:
   void emitShapeAdded(ShapeAnnotation *pShapeAnnotation, GraphicsView *pGraphicsView);
   void emitComponentAdded(Component *pComponent);
   void emitConnectionAdded(LineAnnotation *pConnectionLineAnnotation) {emit connectionAdded(pConnectionLineAnnotation);}
+  void emitCoOrdinateSystemUpdated(GraphicsView *pGraphicsView) {emit coOrdinateSystemUpdated(pGraphicsView);}
 
   OMCInterface::getClassInformation_res mClassInformation;
 private:
@@ -166,8 +173,6 @@ private:
   bool mReadOnly;
   bool mIsSaved;
   SaveContentsType mSaveContentsType;
-  QString mToolTip;
-  QIcon mIcon;
   QPixmap mPixmap;
   QPixmap mDragPixmap;
   QString mClassTextBefore;
@@ -186,6 +191,7 @@ signals:
   void componentAddedForComponent();
   void connectionAdded(LineAnnotation *pConnectionLineAnnotation);
   void iconUpdated();
+  void coOrdinateSystemUpdated(GraphicsView *pGraphicsView);
 public slots:
   void handleLoaded(LibraryTreeItem *pLibraryTreeItem);
   void handleUnloaded();
@@ -193,6 +199,7 @@ public slots:
   void handleComponentAdded(Component *pComponent);
   void handleConnectionAdded(LineAnnotation *pConnectionLineAnnotation);
   void handleIconUpdated();
+  void handleCoOrdinateSystemUpdated(GraphicsView *pGraphicsView);
 };
 
 class LibraryWidget;
@@ -200,9 +207,10 @@ class LibraryTreeProxyModel : public QSortFilterProxyModel
 {
   Q_OBJECT
 public:
-  LibraryTreeProxyModel(LibraryWidget *pLibraryWidget);
+  LibraryTreeProxyModel(LibraryWidget *pLibraryWidget, bool showOnlyModelica);
 private:
   LibraryWidget *mpLibraryWidget;
+  bool mShowOnlyModelica;
 protected:
   virtual bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const;
 };
@@ -220,19 +228,20 @@ public:
   QModelIndex parent(const QModelIndex & index) const;
   QVariant data(const QModelIndex & index, int role = Qt::DisplayRole) const;
   Qt::ItemFlags flags(const QModelIndex &index) const;
-  LibraryTreeItem* findLibraryTreeItem(const QString &name, LibraryTreeItem *root = 0, Qt::CaseSensitivity caseSensitivity = Qt::CaseSensitive) const;
-  LibraryTreeItem* findLibraryTreeItem(const QRegExp &regExp, LibraryTreeItem *root = 0) const;
+  LibraryTreeItem* findLibraryTreeItem(const QString &name, LibraryTreeItem *pLibraryTreeItem = 0,
+                                       Qt::CaseSensitivity caseSensitivity = Qt::CaseSensitive) const;
+  LibraryTreeItem* findLibraryTreeItem(const QRegExp &regExp, LibraryTreeItem *pLibraryTreeItem = 0) const;
+  LibraryTreeItem* findLibraryTreeItemOneLevel(const QString &name, LibraryTreeItem *pLibraryTreeItem = 0,
+                                               Qt::CaseSensitivity caseSensitivity = Qt::CaseSensitive) const;
   LibraryTreeItem* findNonExistingLibraryTreeItem(const QString &name, Qt::CaseSensitivity caseSensitivity = Qt::CaseSensitive) const;
   QModelIndex libraryTreeItemIndex(const LibraryTreeItem *pLibraryTreeItem) const;
-  void addModelicaLibraries(QSplashScreen *pSplashScreen);
-  void createLibraryTreeItems(LibraryTreeItem *pLibraryTreeItem);
+  void addModelicaLibraries();
   LibraryTreeItem* createLibraryTreeItem(QString name, LibraryTreeItem *pParentLibraryTreeItem, bool isSaved = true,
                                          bool isSystemLibrary = false, bool load = false, int row = -1);
-  LibraryTreeItem* createLibraryTreeItem(LibraryTreeItem::LibraryType type, QString name, bool isSaved);
   LibraryTreeItem* createNonExistingLibraryTreeItem(QString nameStructure);
-  void createNonExistingLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem, LibraryTreeItem *pParentLibraryTreeItem, bool isSaved = true,
-                                        int row = -1);
-  void loadNonExistingLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
+  void createLibraryTreeItems(QFileInfo fileInfo, LibraryTreeItem *pParentLibraryTreeItem);
+  LibraryTreeItem* createLibraryTreeItem(LibraryTreeItem::LibraryType type, QString name, QString nameStructure, QString path, bool isSaved,
+                                         LibraryTreeItem *pParentLibraryTreeItem, int row = -1);
   void checkIfAnyNonExistingClassLoaded();
   void addNonExistingLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem) {mNonExistingLibraryTreeItemsList.append(pLibraryTreeItem);}
   void removeNonExistingLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem) {mNonExistingLibraryTreeItemsList.removeOne(pLibraryTreeItem);}
@@ -244,15 +253,17 @@ public:
   void loadLibraryTreeItemPixmap(LibraryTreeItem *pLibraryTreeItem);
   void loadDependentLibraries(QStringList libraries);
   LibraryTreeItem* getLibraryTreeItemFromFile(QString fileName, int lineNumber);
-  void showModelWidget(LibraryTreeItem *pLibraryTreeItem, QString text = QString(""), bool show = true);
+  void showModelWidget(LibraryTreeItem *pLibraryTreeItem, bool show = true);
   void showHideProtectedClasses();
   bool unloadClass(LibraryTreeItem *pLibraryTreeItem, bool askQuestion = true);
-  bool unloadTLMOrTextFile(LibraryTreeItem *pLibraryTreeItem, bool askQuestion = true);
-  bool unloadLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem, bool deleteClass = false);
+  bool unloadCompositeModelOrTextFile(LibraryTreeItem *pLibraryTreeItem, bool askQuestion = true);
+  bool unloadLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem, bool doDeleteClass);
   bool removeLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
+  bool deleteTextFile(LibraryTreeItem *pLibraryTreeItem, bool askQuestion = true);
   void moveClassUpDown(LibraryTreeItem *pLibraryTreeItem, bool up);
   void moveClassTopBottom(LibraryTreeItem *pLibraryTreeItem, bool top);
   void updateBindings(LibraryTreeItem *pLibraryTreeItem);
+  void generateVerificationScenarios(LibraryTreeItem *pLibraryTreeItem);
   QString getUniqueTopLevelItemName(QString name, int number = 1);
   void emitDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight) {emit dataChanged(topLeft, bottomRight);}
 private:
@@ -265,8 +276,20 @@ private:
   void updateChildLibraryTreeItemClassText(LibraryTreeItem *pLibraryTreeItem, QString contents, QString fileName);
   void readLibraryTreeItemClassTextFromText(LibraryTreeItem *pLibraryTreeItem, QString contents);
   QString readLibraryTreeItemClassTextFromFile(LibraryTreeItem *pLibraryTreeItem);
+  void createLibraryTreeItems(LibraryTreeItem *pLibraryTreeItem);
+  LibraryTreeItem* createLibraryTreeItemImpl(QString name, LibraryTreeItem *pParentLibraryTreeItem, bool isSaved = true,
+                                             bool isSystemLibrary = false, bool load = false, int row = -1);
+  void createNonExistingLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem, LibraryTreeItem *pParentLibraryTreeItem, bool isSaved = true,
+                                        int row = -1);
+  void createLibraryTreeItemsImpl(QFileInfo fileInfo, LibraryTreeItem *pParentLibraryTreeItem);
+  LibraryTreeItem* createLibraryTreeItemImpl(LibraryTreeItem::LibraryType type, QString name, QString nameStructure, QString path, bool isSaved,
+                                             LibraryTreeItem *pParentLibraryTreeItem, int row = -1);
   void unloadClassHelper(LibraryTreeItem *pLibraryTreeItem, LibraryTreeItem *pParentLibraryTreeItem);
   void unloadClassChildren(LibraryTreeItem *pLibraryTreeItem);
+  void unloadFileHelper(LibraryTreeItem *pLibraryTreeItem, LibraryTreeItem *pParentLibraryTreeItem);
+  void unloadFileChildren(LibraryTreeItem *pLibraryTreeItem);
+  void deleteFileHelper(LibraryTreeItem *pLibraryTreeItem, LibraryTreeItem *pParentLibraryTreeItem);
+  void deleteFileChildren(LibraryTreeItem *pLibraryTreeItem);
 protected:
   Qt::DropActions supportedDropActions() const;
 };
@@ -281,6 +304,7 @@ private:
   LibraryWidget *mpLibraryWidget;
   QAction *mpOpenClassAction;
   QAction *mpViewDocumentationAction;
+  QAction *mpInformationAction;
   QAction *mpNewModelicaClassAction;
   QAction *mpSaveAction;
   QAction *mpSaveAsAction;
@@ -296,14 +320,22 @@ private:
   QAction *mpSimulateAction;
   QAction *mpSimulateWithTransformationalDebuggerAction;
   QAction *mpSimulateWithAlgorithmicDebuggerAction;
+#if !defined(WITHOUT_OSG)
+  QAction *mpSimulateWithAnimationAction;
+#endif
   QAction *mpSimulationSetupAction;
   QAction *mpDuplicateClassAction;
   QAction *mpUnloadClassAction;
-  QAction *mpUnloadTLMFileAction;
+  QAction *mpUnloadCompositeModelFileAction;
+  QAction *mpNewFileAction;
+  QAction *mpNewFolderAction;
+  QAction *mpRenameAction;
+  QAction *mpDeleteAction;
   QAction *mpExportFMUAction;
   QAction *mpExportXMLAction;
   QAction *mpExportFigaroAction;
   QAction *mpUpdateBindingsAction;
+  QAction *mpGenerateVerificationScenariosAction;
   QAction *mpFetchInterfaceDataAction;
   QAction *mpTLMCoSimulationAction;
   void createActions();
@@ -314,6 +346,7 @@ public slots:
   void showContextMenu(QPoint point);
   void openClass();
   void viewDocumentation();
+  void openInformationDialog();
   void createNewModelicaClass();
   void saveClass();
   void saveAsClass();
@@ -328,14 +361,20 @@ public slots:
   void simulate();
   void simulateWithTransformationalDebugger();
   void simulateWithAlgorithmicDebugger();
+  void simulateWithAnimation();
   void simulationSetup();
   void duplicateClass();
   void unloadClass();
-  void unloadTLMOrTextFile();
+  void unloadCompositeModelOrTextFile();
+  void createNewFile();
+  void createNewFolder();
+  void renameLibraryTreeItem();
+  void deleteTextFile();
   void exportModelFMU();
   void exportModelXML();
   void exportModelFigaro();
   void updateBindings();
+  void generateVerificationScenarios();
   void fetchInterfaceData();
   void TLMSimulate();
 protected:
@@ -344,32 +383,32 @@ protected:
   virtual void keyPressEvent(QKeyEvent *event);
 };
 
-class MainWindow;
 class LibraryWidget : public QWidget
 {
   Q_OBJECT
 public:
-  LibraryWidget(MainWindow *pMainWindow);
-  MainWindow* getMainWindow() {return mpMainWindow;}
+  LibraryWidget(QWidget *pParent = 0);
   TreeSearchFilters* getTreeSearchFilters() {return mpTreeSearchFilters;}
   LibraryTreeModel* getLibraryTreeModel() {return mpLibraryTreeModel;}
   LibraryTreeProxyModel* getLibraryTreeProxyModel() {return mpLibraryTreeProxyModel;}
   LibraryTreeView* getLibraryTreeView() {return mpLibraryTreeView;}
-  void openFile(QString fileName, QString encoding = Helper::utf8, bool showProgress = true, bool checkFileExists = false);
+  void openFile(QString fileName, QString encoding = Helper::utf8, bool showProgress = true, bool checkFileExists = false,
+                bool loadExternalModel = false);
   void openModelicaFile(QString fileName, QString encoding = Helper::utf8, bool showProgress = true);
-  void openTLMOrTextFile(QFileInfo fileInfo, bool showProgress = true);
+  void openCompositeModelOrTextFile(QFileInfo fileInfo, bool showProgress = true);
+  void openDirectory(QFileInfo fileInfo, bool showProgress = true);
+  bool parseCompositeModelFile(QFileInfo fileInfo, QString *pCompositeModelName);
   void parseAndLoadModelicaText(QString modelText);
+  bool saveFile(QString fileName, QString contents);
   bool saveLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
   void saveAsLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
   bool saveTotalLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
   void openLibraryTreeItem(QString nameStructure);
 private:
-  MainWindow *mpMainWindow;
   TreeSearchFilters *mpTreeSearchFilters;
   LibraryTreeModel *mpLibraryTreeModel;
   LibraryTreeProxyModel *mpLibraryTreeProxyModel;
   LibraryTreeView *mpLibraryTreeView;
-  bool saveFile(QString fileName, QString contents);
   bool saveModelicaLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
   bool saveModelicaLibraryTreeItemHelper(LibraryTreeItem *pLibraryTreeItem);
   bool saveModelicaLibraryTreeItemOneFile(LibraryTreeItem *pLibraryTreeItem);
@@ -377,7 +416,9 @@ private:
   void saveChildLibraryTreeItemsOneFileHelper(LibraryTreeItem *pLibraryTreeItem);
   bool saveModelicaLibraryTreeItemFolder(LibraryTreeItem *pLibraryTreeItem);
   bool saveTextLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
-  bool saveTLMLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
+  bool saveCompositeModelLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
+  bool saveAsCompositeModelLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem);
+  bool saveCompositeModelLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem, QString fileName);
   bool saveTotalLibraryTreeItemHelper(LibraryTreeItem *pLibraryTreeItem);
 public slots:
   void searchClasses();
