@@ -29,32 +29,34 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #ifndef MODELWIDGETCONTAINER_H
 #define MODELWIDGETCONTAINER_H
 
+#include "CoOrdinateSystem.h"
+#include "Component/Component.h"
+#include "Util/StringHandler.h"
+#include "Util/Helper.h"
+#include "Editors/BaseEditor.h"
+#include "Editors/ModelicaEditor.h"
+#include "Editors/CompositeModelEditor.h"
+#include "Editors/CEditor.h"
+#include "Editors/TextEditor.h"
+#include "Editors/MetaModelicaEditor.h"
+#include "LibraryTreeWidget.h"
+
 #include <QGraphicsView>
 #include <QGraphicsScene>
 #include <QStatusBar>
 #include <QListWidget>
 #include <QMdiArea>
-#include <map>
 #include <QtWebKit>
 #include <QtXmlPatterns>
-
-#include "CoOrdinateSystem.h"
-#include "Component.h"
-#include "StringHandler.h"
-#include "Helper.h"
-#include "BaseEditor.h"
-#include "ModelicaTextEditor.h"
-#include "TLMEditor.h"
-#include "TextEditor.h"
+#include <QSplitter>
+#include <QUndoStack>
+#include <QUndoView>
 
 class ModelWidget;
 class ComponentInfo;
@@ -85,22 +87,30 @@ private:
   bool mIsCustomScale;
   bool mAddClassAnnotationNeeded;
   bool mIsCreatingConnection;
+  bool mIsCreatingTransition;
   bool mIsCreatingLineShape;
   bool mIsCreatingPolygonShape;
   bool mIsCreatingRectangleShape;
   bool mIsCreatingEllipseShape;
   bool mIsCreatingTextShape;
   bool mIsCreatingBitmapShape;
+  bool mIsPanning;
+  QPoint mLastMouseEventPos;
   Component *mpClickedComponent;
+  Component *mpClickedState;
   bool mIsMovingComponentsAndShapes;
   bool mRenderingLibraryPixmap;
+  bool mSkipFoucusOutEvent;
   QList<Component*> mComponentsList;
   QList<LineAnnotation*> mConnectionsList;
+  QList<LineAnnotation*> mTransitionsList;
+  QList<LineAnnotation*> mInitialStatesList;
   QList<ShapeAnnotation*> mShapesList;
   QList<Component*> mInheritedComponentsList;
   QList<LineAnnotation*> mInheritedConnectionsList;
   QList<ShapeAnnotation*> mInheritedShapesList;
   LineAnnotation *mpConnectionLineAnnotation;
+  LineAnnotation *mpTransitionLineAnnotation;
   LineAnnotation *mpLineShapeAnnotation;
   PolygonAnnotation *mpPolygonShapeAnnotation;
   RectangleAnnotation *mpRectangleShapeAnnotation;
@@ -108,6 +118,8 @@ private:
   TextAnnotation *mpTextShapeAnnotation;
   BitmapAnnotation *mpBitmapShapeAnnotation;
   QAction *mpPropertiesAction;
+  QAction *mpRenameAction;
+  QAction *mpSimulationParamsAction;
   QAction *mpManhattanizeAction;
   QAction *mpDeleteAction;
   QAction *mpBringToFrontAction;
@@ -119,6 +131,8 @@ private:
   QAction *mpRotateAntiClockwiseAction;
   QAction *mpFlipHorizontalAction;
   QAction *mpFlipVerticalAction;
+  QAction *mpSetInitialStateAction;
+  QAction *mpCancelTransitionAction;
 public:
   GraphicsView(StringHandler::ViewType viewType, ModelWidget *parent);
   CoOrdinateSystem mCoOrdinateSystem;
@@ -133,6 +147,8 @@ public:
   bool isAddClassAnnotationNeeded() {return mAddClassAnnotationNeeded;}
   void setIsCreatingConnection(bool enable);
   bool isCreatingConnection() {return mIsCreatingConnection;}
+  void setIsCreatingTransition(bool enable);
+  bool isCreatingTransition() {return mIsCreatingTransition;}
   void setIsCreatingLineShape(bool enable);
   bool isCreatingLineShape() {return mIsCreatingLineShape;}
   void setIsCreatingPolygonShape(bool enable);
@@ -145,6 +161,9 @@ public:
   bool isCreatingTextShape() {return mIsCreatingTextShape;}
   void setIsCreatingBitmapShape(bool enable);
   bool isCreatingBitmapShape() {return mIsCreatingBitmapShape;}
+  void setIsPanning(bool enable);
+  bool isPanning() {return mIsPanning;}
+  void setDragModeInternal(bool enable, bool updateCursor = false);
   void setItemsFlags(bool enable);
   void updateUndoRedoActions(bool enable);
   void setIsMovingComponentsAndShapes(bool enable) {mIsMovingComponentsAndShapes = enable;}
@@ -165,8 +184,8 @@ public:
   QAction* getFlipHorizontalAction() {return mpFlipHorizontalAction;}
   QAction* getFlipVerticalAction() {return mpFlipVerticalAction;}
   bool addComponent(QString className, QPointF position);
-  void addComponentToView(QString name, LibraryTreeItem *pLibraryTreeItem, QString transformationString, QPointF position,
-                          QStringList dialogAnnotation, ComponentInfo *pComponentInfo, bool addObject = true, bool openingClass = false);
+  void addComponentToView(QString name, LibraryTreeItem *pLibraryTreeItem, QString annotation, QPointF position,
+                          ComponentInfo *pComponentInfo, bool addObject = true, bool openingClass = false);
   void addComponentToList(Component *pComponent) {mComponentsList.append(pComponent);}
   void addInheritedComponentToList(Component *pComponent) {mInheritedComponentsList.append(pComponent);}
   void addComponentToClass(Component *pComponent);
@@ -182,15 +201,26 @@ public:
   QList<LineAnnotation*> getConnectionsList() {return mConnectionsList;}
   QList<LineAnnotation*> getInheritedConnectionsList() {return mInheritedConnectionsList;}
   void addConnectionToClass(LineAnnotation *pConnectionLineAnnotation);
-  void deleteConnectionFromClass(LineAnnotation *pConnectonLineAnnotation);
+  void deleteConnectionFromClass(LineAnnotation *pConnectionLineAnnotation);
+  void updateConnectionInClass(LineAnnotation *pConnectionLineAnnotation);
   void addConnectionToList(LineAnnotation *pConnectionLineAnnotation) {mConnectionsList.append(pConnectionLineAnnotation);}
   void addInheritedConnectionToList(LineAnnotation *pConnectionLineAnnotation) {mInheritedConnectionsList.append(pConnectionLineAnnotation);}
   void deleteConnectionFromList(LineAnnotation *pConnectionLineAnnotation) {mConnectionsList.removeOne(pConnectionLineAnnotation);}
   void deleteInheritedConnectionFromList(LineAnnotation *pConnectionLineAnnotation) {mInheritedConnectionsList.removeOne(pConnectionLineAnnotation);}
-  void addShapeToList(ShapeAnnotation *pShape) {mShapesList.append(pShape);}
+  QList<LineAnnotation*> getTransitionsList() {return mTransitionsList;}
+  void addTransitionToClass(LineAnnotation *pTransitionLineAnnotation);
+  void deleteTransitionFromClass(LineAnnotation *pTransitionLineAnnotation);
+  void addTransitionToList(LineAnnotation *pTransitionLineAnnotation) {mTransitionsList.append(pTransitionLineAnnotation);}
+  void deleteTransitionFromList(LineAnnotation *pTransitionLineAnnotation) {mTransitionsList.removeOne(pTransitionLineAnnotation);}
+  QList<LineAnnotation*> getInitialStatesList() {return mInitialStatesList;}
+  void addInitialStateToClass(LineAnnotation *pInitialStateLineAnnotation);
+  void deleteInitialStateFromClass(LineAnnotation *pInitialStateLineAnnotation);
+  void addInitialStateToList(LineAnnotation *pInitialStateLineAnnotation) {mInitialStatesList.append(pInitialStateLineAnnotation);}
+  void deleteInitialStateFromList(LineAnnotation *pInitialStateLineAnnotation) {mInitialStatesList.removeOne(pInitialStateLineAnnotation);}
+  void addShapeToList(ShapeAnnotation *pShape, int index = -1);
   void addInheritedShapeToList(ShapeAnnotation *pShape) {mInheritedShapesList.append(pShape);}
   void deleteShape(ShapeAnnotation *pShapeAnnotation);
-  void deleteShapeFromList(ShapeAnnotation *pShape) {mShapesList.removeOne(pShape);}
+  int deleteShapeFromList(ShapeAnnotation *pShape);
   void deleteInheritedShapeFromList(ShapeAnnotation *pShape) {mInheritedShapesList.removeOne(pShape);}
   void reOrderShapes();
   void bringToFront(ShapeAnnotation *pShape);
@@ -213,10 +243,13 @@ public:
   bool hasAnnotation();
   void addItem(QGraphicsItem *pGraphicsItem);
   void removeItem(QGraphicsItem *pGraphicsItem);
+  void fitInViewInternal();
 private:
   void createActions();
   bool isClassDroppedOnItself(LibraryTreeItem *pLibraryTreeItem);
   bool isAnyItemSelectedAndEditable(int key);
+  Component* connectorComponentAtPosition(QPoint position);
+  Component* stateComponentAtPosition(QPoint position);
 signals:
   void mouseManhattanize();
   void mouseDelete();
@@ -228,6 +261,8 @@ signals:
   void keyPressDelete();
   void keyPressRotateClockwise();
   void keyPressRotateAntiClockwise();
+  void keyPressFlipHorizontal();
+  void keyPressFlipVertical();
   void keyPressUp();
   void keyPressShiftUp();
   void keyPressCtrlUp();
@@ -245,13 +280,19 @@ public slots:
   void addConnection(Component *pComponent);
   void removeCurrentConnection();
   void deleteConnection(LineAnnotation *pConnectionLineAnnotation);
+  void addTransition(Component *pComponent);
+  void removeCurrentTransition();
+  void deleteTransition(LineAnnotation *pTransitionLineAnnotation);
+  void deleteInitialState(LineAnnotation *pInitialLineAnnotation);
   void resetZoom();
   void zoomIn();
   void zoomOut();
   void selectAll();
   void clearSelection();
-  void addClassAnnotation();
+  void addClassAnnotation(bool alwaysAdd = true);
   void showGraphicsViewProperties();
+  void showRenameDialog();
+  void showSimulationParamsDialog();
   void manhattanizeItems();
   void deleteItems();
   void duplicateItems();
@@ -259,6 +300,8 @@ public slots:
   void rotateAntiClockwise();
   void flipHorizontal();
   void flipVertical();
+  void setInitialState();
+  void cancelTransition();
 protected:
   virtual void dragMoveEvent(QDragMoveEvent *event);
   virtual void dropEvent(QDropEvent *event);
@@ -273,18 +316,18 @@ protected:
   virtual void contextMenuEvent(QContextMenuEvent *event);
   virtual void resizeEvent(QResizeEvent *event);
   virtual void wheelEvent(QWheelEvent *event);
+  virtual void leaveEvent(QEvent *event);
 };
 
 class WelcomePageWidget : public QWidget
 {
   Q_OBJECT
 public:
-  WelcomePageWidget(MainWindow *parent = 0);
+  WelcomePageWidget(QWidget *pParent = 0);
   void addRecentFilesListItems();
   QFrame* getLatestNewsFrame();
   QSplitter* getSplitter();
 private:
-  MainWindow *mpMainWindow;
   QFrame *mpMainFrame;
   QFrame *mpTopFrame;
   Label *mpPixmapLabel;
@@ -314,14 +357,14 @@ private slots:
 };
 
 class ModelWidgetContainer;
-class ModelicaTextHighlighter;
-class TLMHighlighter;
+class ModelicaHighlighter;
+class CompositeModelHighlighter;
 class Label;
 class ModelWidget : public QWidget
 {
   Q_OBJECT
 public:
-  ModelWidget(LibraryTreeItem* pLibraryTreeItem, ModelWidgetContainer *pModelWidgetContainer, QString text);
+  ModelWidget(LibraryTreeItem* pLibraryTreeItem, ModelWidgetContainer *pModelWidgetContainer);
   ModelWidgetContainer* getModelWidgetContainer() {return mpModelWidgetContainer;}
   void setLibraryTreeItem(LibraryTreeItem *pLibraryTreeItem) {mpLibraryTreeItem = pLibraryTreeItem;}
   LibraryTreeItem* getLibraryTreeItem() {return mpLibraryTreeItem;}
@@ -333,34 +376,42 @@ public:
   GraphicsView* getIconGraphicsView() {return mpIconGraphicsView;}
   QUndoStack* getUndoStack() {return mpUndoStack;}
   BaseEditor* getEditor() {return mpEditor;}
+  void setModelClassPathLabel(QString path) {mpModelClassPathLabel->setText(path);}
   void setModelFilePathLabel(QString path) {mpModelFilePathLabel->setText(path);}
-  Label* getCursorPositionLabel() {return mpCursorPositionLabel;}
   bool isLoadedWidgetComponents() {return mCreateModelWidgetComponents;}
   void addInheritedClass(LibraryTreeItem *pLibraryTreeItem) {mInheritedClassesList.append(pLibraryTreeItem);}
   void removeInheritedClass(LibraryTreeItem *pLibraryTreeItem) {mInheritedClassesList.removeOne(pLibraryTreeItem);}
   void clearInheritedClasses() {mInheritedClassesList.clear();}
   QList<LibraryTreeItem*> getInheritedClassesList() {return mInheritedClassesList;}
-  QMap<QString, QMap<QString, QString> > getExtendsModifiersMap() {return mExtendsModifiersMap;}
   QMap<QString, QString> getExtendsModifiersMap(QString extendsClass);
   void fetchExtendsModifiers(QString extendsClass);
   void reDrawModelWidgetInheritedClasses();
+  void drawBaseCoOrdinateSystem(ModelWidget *pModelWidget, GraphicsView *pGraphicsView);
   ShapeAnnotation* createNonExistingInheritedShape(GraphicsView *pGraphicsView);
   ShapeAnnotation* createInheritedShape(ShapeAnnotation *pShapeAnnotation, GraphicsView *pGraphicsView);
   Component* createInheritedComponent(Component *pComponent, GraphicsView *pGraphicsView);
   LineAnnotation* createInheritedConnection(LineAnnotation *pConnectionLineAnnotation);
+  void loadComponents();
   void loadDiagramView();
   void loadConnections();
   void createModelWidgetComponents();
   Component* getConnectorComponent(Component *pConnectorComponent, QString connectorName);
+  void clearGraphicsViews();
   void reDrawModelWidget();
-  bool validateText();
-  bool modelicaEditorTextChanged();
+  bool validateText(LibraryTreeItem **pLibraryTreeItem);
+  bool modelicaEditorTextChanged(LibraryTreeItem **pLibraryTreeItem);
   void updateChildClasses(LibraryTreeItem *pLibraryTreeItem);
   void clearSelection();
   void updateClassAnnotationIfNeeded();
-  void updateModelicaText();
+  void updateModelText(bool updateText = true);
   void updateModelicaTextManually(QString contents);
   void updateUndoRedoActions();
+  void updateDynamicResults(QString resultFileName);
+  QString getResultFileName() {return mResultFileName;}
+  bool writeCoSimulationResultFile(QString fileName);
+  bool writeVisualXMLFile(QString fileName, bool canWriteVisualXMLFile = false);
+  void beginMacro(const QString &text);
+  void endMacro();
 private:
   ModelWidgetContainer *mpModelWidgetContainer;
   LibraryTreeItem *mpLibraryTreeItem;
@@ -372,8 +423,8 @@ private:
   Label *mpReadOnlyLabel;
   Label *mpModelicaTypeLabel;
   Label *mpViewTypeLabel;
+  Label *mpModelClassPathLabel;
   Label *mpModelFilePathLabel;
-  Label *mpCursorPositionLabel;
   QToolButton *mpFileLockToolButton;
   GraphicsView *mpDiagramGraphicsView;
   GraphicsScene *mpDiagramGraphicsScene;
@@ -382,9 +433,8 @@ private:
   QUndoStack *mpUndoStack;
   QUndoView *mpUndoView;
   BaseEditor *mpEditor;
-  ModelicaTextHighlighter *mpModelicaTextHighlighter;
-  TLMHighlighter *mpTLMHighlighter;
   QStatusBar *mpModelStatusBar;
+  bool mComponentsLoaded;
   bool mDiagramViewLoaded;
   bool mConnectionsLoaded;
   bool mCreateModelWidgetComponents;
@@ -393,6 +443,7 @@ private:
   QList<LibraryTreeItem*> mInheritedClassesList;
   QList<ComponentInfo*> mComponentsList;
   QStringList mComponentsAnnotationsList;
+  QString mResultFileName;
 
   void getModelInheritedClasses();
   void drawModelInheritedClassShapes(ModelWidget *pModelWidget, StringHandler::ViewType viewType);
@@ -400,14 +451,21 @@ private:
   void getModelIconDiagramShapes(StringHandler::ViewType viewType);
   void drawModelInheritedClassComponents(ModelWidget *pModelWidget, StringHandler::ViewType viewType);
   void removeInheritedClassComponents(StringHandler::ViewType viewType);
+  void removeClassComponents(StringHandler::ViewType viewType);
   void getModelComponents();
   void drawModelIconComponents();
   void drawModelDiagramComponents();
   void drawModelInheritedClassConnections(ModelWidget *pModelWidget);
   void removeInheritedClassConnections();
   void getModelConnections();
-  void getTLMComponents();
-  void getTLMConnections();
+  void getModelTransitions();
+  void getModelInitialStates();
+  void getMetaModelSubModels();
+  void getMetaModelConnections();
+  void detectMultipleDeclarations();
+  QString getCompositeModelName();
+  void getCompositeModelSubModels();
+  void getCompositeModelConnections();
 private slots:
   void showIconView(bool checked);
   void showDiagramView(bool checked);
@@ -415,29 +473,36 @@ private slots:
 public slots:
   void makeFileWritAble();
   void showDocumentationView();
-  bool TLMEditorTextChanged();
+  bool compositeModelEditorTextChanged();
   void handleCanUndoChanged(bool canUndo);
   void handleCanRedoChanged(bool canRedo);
+  void removeDynamicResults(QString resultFileName = "");
 protected:
   virtual void closeEvent(QCloseEvent *event);
 };
 
-class MdiArea;
-class ModelWidgetContainer : public MdiArea
+
+void addCloseActionsToSubWindowSystemMenu(QMdiSubWindow *pMdiSubWindow);
+
+class ModelWidgetContainer : public QMdiArea
 {
   Q_OBJECT
 public:
-  ModelWidgetContainer(MainWindow *pParent);
+  ModelWidgetContainer(QWidget *pParent = 0);
   void addModelWidget(ModelWidget *pModelWidget, bool checkPreferedView = true);
   ModelWidget* getCurrentModelWidget();
+  ModelWidget* getModelWidget(const QString &className);
   QMdiSubWindow* getCurrentMdiSubWindow();
   QMdiSubWindow* getMdiSubWindow(ModelWidget *pModelWidget);
-  void setPreviousViewType(StringHandler::ViewType viewType);
-  StringHandler::ViewType getPreviousViewType();
-  void setShowGridLines(bool On);
-  bool isShowGridLines();
+  void setPreviousViewType(StringHandler::ViewType viewType) {mPreviousViewType = viewType;}
+  StringHandler::ViewType getPreviousViewType() {return mPreviousViewType;}
+  void setShowGridLines(bool On) {mShowGridLines = On;}
+  bool isShowGridLines() {return mShowGridLines;}
   bool eventFilter(QObject *object, QEvent *event);
   void changeRecentModelsListSelection(bool moveDown);
+#if !defined(WITHOUT_OSG)
+  void updateThreeDViewer(ModelWidget *pModelWidget);
+#endif
 private:
   StringHandler::ViewType mPreviousViewType;
   bool mShowGridLines;
@@ -445,12 +510,15 @@ private:
   QListWidget *mpRecentModelsList;
   void loadPreviousViewType(ModelWidget *pModelWidget);
 public slots:
-  void openRecentModelWidget(QListWidgetItem *pItem);
+  bool openRecentModelWidget(QListWidgetItem *pListWidgetItem);
   void currentModelWidgetChanged(QMdiSubWindow *pSubWindow);
+  void updateThreeDViewer(QMdiSubWindow *pSubWindow);
   void saveModelWidget();
   void saveAsModelWidget();
   void saveTotalModelWidget();
   void printModel();
+  void showSimulationParams();
+  void alignInterfaces();
 };
 
 #endif // MODELWIDGETCONTAINER_H
