@@ -34,6 +34,7 @@
 
 #include "meta/meta_modelica.h"
 #include "omc_config.h"
+#include "gc.h"
 
 extern "C" {
 void (*omc_assert)(threadData_t*,FILE_INFO info,const char *msg,...) __attribute__((noreturn)) = omc_assert_function;
@@ -43,6 +44,7 @@ void (*omc_throw)(threadData_t*) __attribute__ ((noreturn)) = omc_throw_function
 int omc_Main_handleCommand(void *threadData, void *imsg, void *ist, void **omsg, void **ost);
 void* omc_Main_init(void *threadData, void *args);
 void* omc_Main_readSettings(void *threadData, void *args);
+void omc_System_initGarbageCollector(void *threadData);
 #ifdef WIN32
 void omc_Main_setWindowsPaths(threadData_t *threadData, void* _inOMHome);
 #endif
@@ -183,19 +185,11 @@ bool OMCProxy::initializeOMC()
   /* create the tmp path */
   QString& tmpPath = Utilities::tempDirectory();
   /* create a file to write OMEdit communication log */
-  mCommunicationLogFile.setFileName(QString("%1omeditcommunication.log").arg(tmpPath));
-  if (mCommunicationLogFile.open(QIODevice::WriteOnly | QIODevice::Text)) {
-    mCommunicationLogFileTextStream.setDevice(&mCommunicationLogFile);
-    mCommunicationLogFileTextStream.setCodec(Helper::utf8.toStdString().data());
-    mCommunicationLogFileTextStream.setGenerateByteOrderMark(false);
-  }
+  QString communicationLogFilePath = QString("%1omeditcommunication.log").arg(tmpPath);
+  mpCommunicationLogFile = fopen(communicationLogFilePath.toStdString().c_str(), "w");
   /* create a file to write OMEdit commands */
-  mCommandsMosFile.setFileName(QString("%1omeditcommands.mos").arg(tmpPath));
-  if (mCommandsMosFile.open(QIODevice::WriteOnly | QIODevice::Text)) {
-    mCommandsLogFileTextStream.setDevice(&mCommandsMosFile);
-    mCommandsLogFileTextStream.setCodec(Helper::utf8.toStdString().data());
-    mCommandsLogFileTextStream.setGenerateByteOrderMark(false);
-  }
+  QString commandsLogFilePath = QString("%1omeditcommands.mos").arg(tmpPath);
+  mpCommandsLogFile = fopen(commandsLogFilePath.toStdString().c_str(), "w");
   // read the locale
   QSettings *pSettings = Utilities::getApplicationSettings();
   QLocale settingsLocale = QLocale(pSettings->value("language").toString());
@@ -204,7 +198,8 @@ bool OMCProxy::initializeOMC()
   QString locale = "+locale=" + settingsLocale.name();
   args = mmc_mk_cons(mmc_mk_scon(locale.toStdString().c_str()), args);
   // initialize threadData
-  threadData_t *threadData = (threadData_t *) calloc(1, sizeof(threadData_t));
+  omc_System_initGarbageCollector(NULL);
+  threadData_t *threadData = (threadData_t *) GC_malloc(sizeof(threadData_t));
   void *st = 0;
   MMC_TRY_TOP_INTERNAL()
   omc_Main_init(threadData, args);
@@ -233,21 +228,24 @@ bool OMCProxy::initializeOMC()
   changeDirectory(tmpPath);
   // set the OpenModelicaLibrary variable.
   Helper::OpenModelicaLibrary = getModelicaPath();
-  /* Disable code generation by default in OMEdit only (for 1.11.
-   * Will be changed in OMC default for 1.12 */
-  setCommandLineOptions("-d=nogen");
   return true;
 }
 
 /*!
-  Stops the OpenModelica Compiler. Kill the process omc and also deletes the CORBA reference file.
-  \see startServer
-  */
+ * \brief OMCProxy::quitOMC
+ * Quits the OpenModelica Compiler.\n
+ * Closes the log files.
+ * \see startServer
+ */
 void OMCProxy::quitOMC()
 {
   sendCommand("quit()");
-  mCommunicationLogFile.close();
-  mCommandsMosFile.close();
+  if (mpCommunicationLogFile) {
+    fclose(mpCommunicationLogFile);
+  }
+  if (mpCommandsLogFile) {
+    fclose(mpCommandsLogFile);
+  }
 }
 
 /*!
@@ -334,15 +332,15 @@ void OMCProxy::logCommand(QString command, QTime *commandTime)
   mCurrentCommandIndex = mCommandsList.count();
   mpExpressionTextBox->setText("");
   // write the log to communication log file
-  if (mCommunicationLogFileTextStream.device()) {
-    mCommunicationLogFileTextStream << QString("%1 %2\n").arg(command).arg(commandTime->currentTime().toString("hh:mm:ss:zzz"));
+  if (mpCommunicationLogFile) {
+    fputs(QString("%1 %2\n").arg(command, commandTime->currentTime().toString("hh:mm:ss:zzz")).toStdString().c_str(), mpCommunicationLogFile);
   }
   // write commands mos file
-  if (mCommandsLogFileTextStream.device()) {
+  if (mpCommandsLogFile) {
     if (command.compare("quit()") == 0) {
-      mCommandsLogFileTextStream << QString("%1;\n").arg(command);
+      fputs(QString("%1;\n").arg(command).toStdString().c_str(), mpCommandsLogFile);
     } else {
-      mCommandsLogFileTextStream << QString("%1; getErrorString();\n").arg(command);
+      fputs(QString("%1; getErrorString();\n").arg(command).toStdString().c_str(), mpCommandsLogFile);
     }
   }
 }
@@ -362,11 +360,10 @@ void OMCProxy::logResponse(QString response, QTime *responseTime)
   format.setFont(font);
   Utilities::insertText(mpOMCLoggerTextBox, response + "\n\n", format);
   // write the log to communication log file
-  if (mCommunicationLogFileTextStream.device()) {
-    mCommunicationLogFileTextStream << QString("%1 %2\n").arg(response).arg(responseTime->currentTime().toString("hh:mm:ss:zzz"));
+  if (mpCommunicationLogFile) {
+    fputs(QString("%1 %2\n").arg(response).arg(responseTime->currentTime().toString("hh:mm:ss:zzz")).toStdString().c_str(), mpCommunicationLogFile);
     mTotalOMCCallsTime += (double)responseTime->elapsed() / 1000;
-    mCommunicationLogFileTextStream << QString("%1 secs (%2 secs)\n\n").arg(QString::number((double)responseTime->elapsed() / 1000))
-                                       .arg(QString::number(mTotalOMCCallsTime));
+    fputs(QString("%1 secs (%2 secs)\n\n").arg(QString::number((double)responseTime->elapsed() / 1000)).arg(QString::number(mTotalOMCCallsTime)).toStdString().c_str(), mpCommunicationLogFile);
   }
 }
 
@@ -964,7 +961,7 @@ bool OMCProxy::setComponentModifierValue(QString className, QString modifierName
     expression = QString("setComponentModifierValue(%1, %2, $Code(=%3))").arg(className).arg(modifierName).arg(modifierValue);
   }
   sendCommand(expression);
-  if (getResult().toLower().contains("ok")) {
+  if (getResult().toLower().compare("ok") == 0) {
     return true;
   } else {
     QString msg = tr("Unable to set the component modifier value using command <b>%1</b>").arg(expression);
@@ -1030,7 +1027,7 @@ bool OMCProxy::setExtendsModifierValue(QString className, QString extendsClassNa
         .arg(modifierValue);
   }
   sendCommand(expression);
-  if (getResult().toLower().contains("ok")) {
+  if (getResult().toLower().compare("ok") == 0) {
     return true;
   } else {
     QString msg = tr("Unable to set the extends modifier value using command <b>%1</b>").arg(expression);
@@ -1074,7 +1071,9 @@ QString OMCProxy::getIconAnnotation(QString className)
 {
   QString expression = "getIconAnnotation(" + className + ")";
   sendCommand(expression);
-  return getResult();
+  QString result = getResult();
+  printMessagesStringInternal();
+  return result;
 }
 
 /*!
@@ -1086,7 +1085,9 @@ QString OMCProxy::getDiagramAnnotation(QString className)
 {
   QString expression = "getDiagramAnnotation(" + className + ")";
   sendCommand(expression);
-  return getResult();
+  QString result = getResult();
+  printMessagesStringInternal();
+  return result;
 }
 
 /*!
@@ -1096,32 +1097,19 @@ QString OMCProxy::getDiagramAnnotation(QString className)
   */
 int OMCProxy::getConnectionCount(QString className)
 {
-  QString expression = "getConnectionCount(" + className + ")";
-  sendCommand(expression);
-  QString result = getResult();
-  if (!result.isEmpty()) {
-    bool ok;
-    int result_number = result.toInt(&ok);
-    if (ok) {
-      return result_number;
-    } else {
-      return 0;
-    }
-  }
-  return 0;
+  return mpOMCInterface->getConnectionCount(className);
 }
 
 /*!
-  Returns the connection at a specific index from a model.
-  \param className - is the name of the model.
-  \param num - is the index of connection.
-  \return the connection
-  */
-QString OMCProxy::getNthConnection(QString className, int num)
+ * \brief OMCProxy::getNthConnection
+ * Returns the connection at a specific index from a model.
+ * \param className - is the name of the model.
+ * \param index - is the index of connection.
+ * \return the connection list i.e, {from, to, comment}
+ */
+QList<QString> OMCProxy::getNthConnection(QString className, int index)
 {
-  QString expression = "getNthConnection(" + className + ", " + QString::number(num) + ")";
-  sendCommand(expression);
-  return getResult();
+  return mpOMCInterface->getNthConnection(className, index);
 }
 
 /*!
@@ -1137,6 +1125,32 @@ QString OMCProxy::getNthConnectionAnnotation(QString className, int num)
   return getResult();
 }
 
+/*!
+ * \brief OMCProxy::getTransitions
+ * Returns the list of transitions in a class.
+ * \param className
+ * \return
+ */
+QList<QList<QString> > OMCProxy::getTransitions(QString className)
+{
+  QList<QList<QString> > transitions = mpOMCInterface->getTransitions(className);
+  printMessagesStringInternal();
+  return transitions;
+}
+
+/*!
+ * \brief OMCProxy::getInitialStates
+ * Returns the list of initial states in a class.
+ * \param className
+ * \return
+ */
+QList<QList<QString> > OMCProxy::getInitialStates(QString className)
+{
+  QList<QList<QString> > initialStates = mpOMCInterface->getInitialStates(className);
+  printMessagesStringInternal();
+  return initialStates;
+}
+
 /*!
  * \brief OMCProxy::getInheritanceCount
  * Returns the inheritance count of a model.
@@ -1717,10 +1731,11 @@ QString OMCProxy::getDefaultComponentPrefixes(QString className)
 bool OMCProxy::addComponent(QString name, QString className, QString componentName, QString placementAnnotation)
 {
   sendCommand("addComponent(" + name + ", " + className + "," + componentName + "," + placementAnnotation + ")");
-  if (StringHandler::unparseBool(getResult()))
+  if (StringHandler::unparseBool(getResult())) {
     return true;
-  else
+  } else {
     return false;
+  }
 }
 
 /*!
@@ -1732,10 +1747,11 @@ bool OMCProxy::addComponent(QString name, QString className, QString componentNa
 bool OMCProxy::deleteComponent(QString name, QString componentName)
 {
   sendCommand("deleteComponent(" + name + "," + componentName + ")");
-  if (StringHandler::unparseBool(getResult()))
+  if (StringHandler::unparseBool(getResult())) {
     return true;
-  else
+  } else {
     return false;
+  }
 }
 
 /*!
@@ -1750,10 +1766,11 @@ bool OMCProxy::deleteComponent(QString name, QString componentName)
 bool OMCProxy::renameComponent(QString className, QString oldName, QString newName)
 {
   sendCommand("renameComponent(" + className + "," + oldName + "," + newName + ")");
-  if (getResult().toLower().contains("error"))
+  if (getResult().toLower().contains("error")) {
     return false;
-  else
+  } else {
     return true;
+  }
 }
 
 /*!
@@ -1767,10 +1784,11 @@ bool OMCProxy::renameComponent(QString className, QString oldName, QString newNa
 bool OMCProxy::updateComponent(QString name, QString className, QString componentName, QString placementAnnotation)
 {
   sendCommand("updateComponent(" + name + "," + className + "," + componentName + "," + placementAnnotation + ")");
-  if (StringHandler::unparseBool(getResult()))
+  if (StringHandler::unparseBool(getResult())) {
     return true;
-  else
+  } else {
     return false;
+  }
 }
 
 /*!
@@ -1784,10 +1802,11 @@ bool OMCProxy::updateComponent(QString name, QString className, QString componen
 bool OMCProxy::renameComponentInClass(QString className, QString oldName, QString newName)
 {
   sendCommand("renameComponentInClass(" + className + "," + oldName + "," + newName + ")");
-  if (getResult().toLower().contains("error"))
+  if (getResult().toLower().contains("error")) {
     return false;
-  else
+  } else {
     return true;
+  }
 }
 
 /*!
@@ -1801,10 +1820,11 @@ bool OMCProxy::renameComponentInClass(QString className, QString oldName, QStrin
 bool OMCProxy::updateConnection(QString from, QString to, QString className, QString annotation)
 {
   sendCommand("updateConnection(" + from + "," + to + "," + className + "," + annotation + ")");
-  if (getResult().contains("Ok"))
+  if (getResult().toLower().compare("ok") == 0) {
     return true;
-  else
+  } else {
     return false;
+  }
 }
 
 /*!
@@ -1860,7 +1880,7 @@ bool OMCProxy::setComponentComment(QString className, QString componentName, QSt
 bool OMCProxy::setComponentDimensions(QString className, QString componentName, QString dimensions)
 {
   sendCommand("setComponentDimensions(" + className + "," + componentName + "," + dimensions + ")");
-  if (getResult().contains("Ok")) {
+  if (getResult().toLower().compare("ok") == 0) {
     return true;
   } else {
     return false;
@@ -1882,7 +1902,7 @@ bool OMCProxy::addConnection(QString from, QString to, QString className, QStrin
   } else {
     sendCommand("addConnection(" + from + "," + to + "," + className + "," + annotation + ")");
   }
-  if (getResult().contains("Ok")) {
+  if (getResult().toLower().compare("ok") == 0) {
     return true;
   } else {
     return false;
@@ -1899,10 +1919,149 @@ bool OMCProxy::addConnection(QString from, QString to, QString className, QStrin
 bool OMCProxy::deleteConnection(QString from, QString to, QString className)
 {
   sendCommand("deleteConnection(" + from + "," + to + "," + className + ")");
-  if (getResult().contains("Ok"))
+  if (getResult().toLower().compare("ok") == 0) {
     return true;
-  else
-  {
+  } else {
+    printMessagesStringInternal();
+    return false;
+  }
+}
+
+/*!
+ * \brief OMCProxy::addTransition
+ * Adds a transition
+ * \param className - the name of the class.
+ * \param from - the connection start component name.
+ * \param to - the connection end component name.
+ * \param condition
+ * \param immediate
+ * \param reset
+ * \param synchronize
+ * \param priority
+ * \param annotation
+ * \return true on success.
+ */
+bool OMCProxy::addTransition(QString className, QString from, QString to, QString condition, bool immediate, bool reset, bool synchronize,
+                             int priority, QString annotation)
+{
+  sendCommand(QString("addTransition(%1, \"%2\", \"%3\", \"%4\", %5, %6, %7, %8, %9)").arg(className).arg(from).arg(to)
+              .arg(StringHandler::escapeString(condition)).arg(immediate ? "true" : "false").arg(reset ? "true" : "false")
+              .arg(synchronize ? "true" : "false").arg(priority).arg(annotation));
+  if (StringHandler::unparseBool(getResult())) {
+    return true;
+  } else {
+    printMessagesStringInternal();
+    return false;
+  }
+}
+
+/*!
+ * \brief OMCProxy::deleteTransition
+ * Deletes a transition
+ * \param className - the name of the class.
+ * \param from - the connection start component name.
+ * \param to - the connection end component name.
+ * \param immediate
+ * \param reset
+ * \param synchronize
+ * \param priority
+ * \return true on success.
+ */
+bool OMCProxy::deleteTransition(QString className, QString from, QString to, QString condition, bool immediate, bool reset, bool synchronize,
+                                int priority)
+{
+  bool result = mpOMCInterface->deleteTransition(className, from, to, condition, immediate, reset, synchronize, priority);
+  if (!result) {
+    printMessagesStringInternal();
+  }
+  return result;
+}
+
+/*!
+ * \brief OMCProxy::updateTransition
+ * Updates a transition
+ * \param className - the name of the class.
+ * \param from - the connection start component name.
+ * \param to - the connection end component name.
+ * \param oldCondition
+ * \param oldImmediate
+ * \param oldReset
+ * \param oldSynchronize
+ * \param oldPriority
+ * \param condition
+ * \param immediate
+ * \param reset
+ * \param synchronize
+ * \param priority
+ * \param annotation
+ * \return true on success.
+ */
+bool OMCProxy::updateTransition(QString className, QString from, QString to, QString oldCondition, bool oldImmediate, bool oldReset,
+                                bool oldSynchronize, int oldPriority, QString condition, bool immediate, bool reset, bool synchronize,
+                                int priority, QString annotation)
+{
+  sendCommand(QString("updateTransition(%1, \"%2\", \"%3\", \"%4\", %5, %6, %7, %8, \"%9\", %10, %11, %12, %13, %14)").arg(className).arg(from)
+              .arg(to).arg(StringHandler::escapeString(oldCondition)).arg(oldImmediate ? "true" : "false").arg(oldReset ? "true" : "false")
+              .arg(oldSynchronize ? "true" : "false").arg(oldPriority).arg(StringHandler::escapeString(condition))
+              .arg(immediate ? "true" : "false").arg(reset ? "true" : "false").arg(synchronize ? "true" : "false")
+              .arg(priority).arg(annotation));
+  if (StringHandler::unparseBool(getResult())) {
+    return true;
+  } else {
+    printMessagesStringInternal();
+    return false;
+  }
+}
+
+/*!
+ * \brief OMCProxy::addInitialState
+ * Adds an initial state to the class.
+ * \param className
+ * \param state
+ * \param annotation
+ * \return true on success.
+ */
+bool OMCProxy::addInitialState(QString className, QString state, QString annotation)
+{
+  sendCommand(QString("addInitialState(%1, \"%2\", %3)").arg(className).arg(state).arg(annotation));
+  if (StringHandler::unparseBool(getResult())) {
+    return true;
+  } else {
+    printMessagesStringInternal();
+    return false;
+  }
+}
+
+/*!
+ * \brief OMCProxy::deleteInitialState
+ * Deletes an initial state from the class.
+ * \param className
+ * \param state
+ * \return true on success.
+ */
+bool OMCProxy::deleteInitialState(QString className, QString state)
+{
+  bool result = mpOMCInterface->deleteInitialState(className, state);
+  if (!result) {
+    printMessagesStringInternal();
+  }
+  return result;
+}
+
+/*!
+ * \brief OMCProxy::updateInitialState
+ * Updates an initial state.
+ * \param className
+ * \param state
+ * \param annotation
+ * \return true on success.
+ */
+bool OMCProxy::updateInitialState(QString className, QString state, QString annotation)
+{
+  sendCommand(QString("updateInitialState(%1, \"%2\", %9)").arg(className).arg(state).arg(annotation));
+  if (StringHandler::unparseBool(getResult())) {
+    return true;
+  } else {
     printMessagesStringInternal();
     return false;
   }
@@ -2075,34 +2234,32 @@ OMCInterface::getSimulationOptions_res OMCProxy::getSimulationOptions(QString cl
  * \param platforms
  * \return
  */
-bool OMCProxy::buildModelFMU(QString className, double version, QString type, QString fileNamePrefix, QList<QString> platforms)
+QString OMCProxy::buildModelFMU(QString className, double version, QString type, QString fileNamePrefix, QList<QString> platforms, bool includeResources)
 {
-  bool result = false;
   fileNamePrefix = fileNamePrefix.isEmpty() ? "<default>" : fileNamePrefix;
-  QString res = mpOMCInterface->buildModelFMU(className, QString::number(version), type, fileNamePrefix, platforms);
-  if (res.compare("SimCode: The model " + className + " has been translated to FMU") == 0) {
-    result = true;
+  QString fmuFileName = mpOMCInterface->buildModelFMU(className, QString::number(version), type, fileNamePrefix, platforms, includeResources);
+  if (!fmuFileName.isEmpty()) {
     MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel()->loadDependentLibraries(getClassNames());
   }
   printMessagesStringInternal();
-  return result;
+  return fmuFileName;
 }
 
 /*!
-  Creates the XML of the model.
-  \param className - the name of the class.
-  \return the created XML location
-  */
-bool OMCProxy::translateModelXML(QString className)
+ * \brief OMCProxy::translateModelXML
+ * Creates the XML of the model.
+ * \param className - the name of the class.
+ * \return the created XML location
+ */
+QString OMCProxy::translateModelXML(QString className)
 {
-  bool result = false;
   sendCommand("translateModelXML(" + className + ")");
-  if (StringHandler::unparse(getResult()).compare("SimCode: The model " + className + " has been translated to XML") == 0) {
-    result = true;
+  QString xmlFileName = StringHandler::unparse(getResult());
+  if (!xmlFileName.isEmpty()) {
     MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel()->loadDependentLibraries(getClassNames());
   }
   printMessagesStringInternal();
-  return result;
+  return xmlFileName;
 }
 
 /*!
@@ -2222,7 +2379,7 @@ bool OMCProxy::setIndexReductionMethod(QString method)
 /*!
  * \brief OMCProxy::setCommandLineOptions
  * Sets the OMC flags.
- * \param options - a space separated list fo OMC command line options e.g. +d=initialization +cheapmatchingAlgorithm=3
+ * \param options - a space separated list fo OMC command line options e.g. -d=initialization --cheapmatchingAlgorithm=3
  * \return true on success
  */
 bool OMCProxy::setCommandLineOptions(QString options)
@@ -2409,6 +2566,18 @@ QList<QString> OMCProxy::getDerivedUnits(QString baseUnit)
   return result;
 }
 
+QString OMCProxy::getVersionDateAnnotation(QString className)
+{
+  sendCommand("getNamedAnnotation(" + className + ", versionDate)");
+  return StringHandler::unparse(StringHandler::removeFirstLastCurlBrackets(getResult()));
+}
+
+QString OMCProxy::getVersionBuildAnnotation(QString className)
+{
+  sendCommand("getNamedAnnotation(" + className + ", versionBuild)");
+  return StringHandler::removeFirstLastCurlBrackets(getResult());
+}
+
 /*!
   Gets the DocumentationClass annotation.
   \param className - the name of the class.
@@ -2442,9 +2611,7 @@ QString OMCProxy::getCommandLineOptionsAnnotation(QString className)
 QList<QString> OMCProxy::getAnnotationNamedModifiers(QString className, QString annotation)
 {
   QList<QString> result = mpOMCInterface->getAnnotationNamedModifiers(className, annotation);
-  if (result.isEmpty()) {
-    printMessagesStringInternal();
-  }
+  getErrorString();
   return result;
 }
 
@@ -2709,6 +2876,19 @@ bool OMCProxy::inferBindings(QString className)
   return result;
 }
 
+/*!
+ * \brief OMCProxy::generateVerificationScenarios
+ * Generates the verification scenarios.
+ * \param className
+ * \return
+ */
+bool OMCProxy::generateVerificationScenarios(QString className)
+{
+  bool result = mpOMCInterface->generateVerificationScenarios(className);
+  printMessagesStringInternal();
+  return result;
+}
+
 /*!
  * \brief OMCProxy::getUses
  * Returns the uses annotation.
