@@ -49,7 +49,7 @@ QString Helper::omFileTypes = "Modelica Files (*.mo)";
 QString Helper::omnotebookFileTypes = "OMNotebook Files (*.onb *.onbz *.nb)";
 QString Helper::ngspiceNetlistFileTypes = "ngspice Netlist Files (*.cir *.sp *.spice)";
 QString Helper::imageFileTypes = "SVG (*.svg);;PNG image (*.png);;Windows BMP image (*.bmp);;TIFF (*.tiff)";
-QString Helper::bitmapFileTypes = "PNG image (*.png);Windows BMP image (*.bmp);JPEG (*.jpg *.jpeg)";
+QString Helper::bitmapFileTypes = "Image Files (*.png *.bmp *.jpg *.jpeg);;PNG image (*.png);;Windows BMP image (*.bmp);;JPEG (*.jpg *.jpeg)";
 QString Helper::fmuFileTypes = "FMU Files (*.fmu)";
 QString Helper::xmlFileTypes = "XML Files (*.xml)";
 QString Helper::infoXmlFileTypes = "OM Info Files (*_info.json)";
@@ -179,6 +179,9 @@ QString Helper::OpenModelicaCompilerCLI;
 QString Helper::deleteStr;
 QString Helper::copy;
 QString Helper::paste;
+QString Helper::resetZoom;
+QString Helper::zoomIn;
+QString Helper::zoomOut;
 QString Helper::loading;
 QString Helper::question;
 QString Helper::search;
@@ -186,7 +189,7 @@ QString Helper::unloadClass;
 QString Helper::duplicate;
 QString Helper::duplicateTip;
 QString Helper::unloadClassTip;
-QString Helper::unloadMetaModelOrTextTip;
+QString Helper::unloadCompositeModelOrTextTip;
 QString Helper::refresh;
 QString Helper::simulate;
 QString Helper::simulateTip;
@@ -260,6 +263,9 @@ QString Helper::center;
 QString Helper::right;
 QString Helper::createConnection;
 QString Helper::connectionAttributes;
+QString Helper::createTransition;
+QString Helper::editTransition;
+QString Helper::findVariables;
 QString Helper::filterVariables;
 QString Helper::openClass;
 QString Helper::openClassTip;
@@ -284,6 +290,16 @@ QString Helper::debuggingFileNotSaveInfo;
 QString Helper::algorithmicDebugger;
 QString Helper::debugConfigurations;
 QString Helper::debugConfigurationsTip;
+QString Helper::createGitReposiory;
+QString Helper::createGitReposioryTip;
+QString Helper::logCurrentFile;
+QString Helper::logCurrentFileTip;
+QString Helper::stageCurrentFileForCommit;
+QString Helper::stageCurrentFileForCommitTip;
+QString Helper::unstageCurrentFileFromCommit;
+QString Helper::unstageCurrentFileFromCommitTip;
+QString Helper::commitFiles;
+QString Helper::commitFilesTip;
 QString Helper::resume;
 QString Helper::interrupt;
 QString Helper::exit;
@@ -330,6 +346,14 @@ QString Helper::OMCFlagsTip;
 QString Helper::saveExperimentAnnotation;
 QString Helper::saveOpenModelicaSimulationFlagsAnnotation;
 QString Helper::item;
+QString Helper::bold;
+QString Helper::italic;
+QString Helper::underline;
+QString Helper::condition;
+QString Helper::immediate;
+QString Helper::synchronize;
+QString Helper::priority;
+QString Helper::secs;
 
 void Helper::initHelperVariables()
 {
@@ -402,6 +426,9 @@ void Helper::initHelperVariables()
   Helper::deleteStr = tr("Delete");
   Helper::copy = tr("Copy");
   Helper::paste = tr("Paste");
+  Helper::resetZoom = tr("Reset Zoom");
+  Helper::zoomIn = tr("Zoom In");
+  Helper::zoomOut = tr("Zoom Out");
   Helper::loading = tr("Loading");
   Helper::question = tr("Question");
   Helper::search = tr("Search");
@@ -409,7 +436,7 @@ void Helper::initHelperVariables()
   Helper::duplicateTip = tr("Duplicates the item");
   Helper::unloadClass = tr("Unload");
   Helper::unloadClassTip = tr("Unload the Modelica class");
-  Helper::unloadMetaModelOrTextTip = tr("Unloads the MetaModel/Text file");
+  Helper::unloadCompositeModelOrTextTip = tr("Unloads the CompositeModel/Text file");
   Helper::refresh = tr("Refresh");
   Helper::simulate = tr("Simulate");
   Helper::simulateTip = tr("Simulates the Modelica class");
@@ -483,6 +510,9 @@ void Helper::initHelperVariables()
   Helper::right = tr("Right");
   Helper::createConnection = tr("Create Connection");
   Helper::connectionAttributes = tr("Connection Attributes");
+  Helper::createTransition = tr("Create Transition");
+  Helper::editTransition = tr("Edit Transition");
+  Helper::findVariables = tr("Find Variables");
   Helper::filterVariables = tr("Filter Variables");
   Helper::openClass = tr("Open Class");
   Helper::openClassTip = tr("Opens the class details");
@@ -507,6 +537,16 @@ void Helper::initHelperVariables()
   Helper::algorithmicDebugger = tr("Algorithmic Debugger");
   Helper::debugConfigurations = tr("Debug Configurations");
   Helper::debugConfigurationsTip = tr("Manage debug configurations");
+  Helper::createGitReposiory = tr("Create Repository");
+  Helper::createGitReposioryTip = tr("Create a Git repository");
+  Helper::logCurrentFile = tr("Log Current File");
+  Helper::logCurrentFileTip = tr("Logging current file");
+  Helper::stageCurrentFileForCommit = tr("Stage Current File");
+  Helper::stageCurrentFileForCommitTip = tr("Staging current file for next commit");
+  Helper::unstageCurrentFileFromCommit = tr("Unstage Current File");
+  Helper::unstageCurrentFileFromCommitTip = tr("Unstaging current file from next commit");
+  Helper::commitFiles = tr("Commit");
+  Helper::commitFilesTip = tr("Commiting modified files to the repository");
   Helper::resume = tr("Resume");
   Helper::interrupt = tr("Interrupt");
   Helper::exit = tr("Exit");
@@ -549,10 +589,18 @@ void Helper::initHelperVariables()
   Helper::fixErrorsManually = tr("Fix error(s) manually");
   Helper::revertToLastCorrectVersion = tr("Revert to last correct version");
   Helper::OMCFlags = tr("OMC Flags");
-  Helper::OMCFlagsTip = tr("Space separated list of flags e.g., +d=initialization +cheapmatchingAlgorithm=3");
+  Helper::OMCFlagsTip = tr("Space separated list of flags e.g., -d=initialization --cheapmatchingAlgorithm=3");
   Helper::saveExperimentAnnotation = tr("Save experiment annotation inside model");
   Helper::saveOpenModelicaSimulationFlagsAnnotation = tr("Save __OpenModelica_simulationFlags annotation inside model");
   Helper::item = tr("item");
+  Helper::bold = tr("Bold");
+  Helper::italic = tr("Italic");
+  Helper::underline = tr("Underline");
+  Helper::condition = tr("Condition:");
+  Helper::immediate = tr("Immediate");
+  Helper::synchronize = tr("Synchronize");
+  Helper::priority = tr("Priority:");
+  Helper::secs = tr("secs");
 }
 
 QString GUIMessages::getMessage(int type)
@@ -562,7 +610,7 @@ QString GUIMessages::getMessage(int type)
     case CHECK_MESSAGES_BROWSER:
       return tr("Please check the Messages Browser for more error specific details.");
     case SAME_COMPONENT_NAME:
-      return tr("A Component with the name <b>%1</b> already exists. Please choose another name.");
+      return tr("A component with the name <b>%1</b> already exists. Please choose another name.");
     case SAME_COMPONENT_CONNECT:
       return tr("You cannot connect a component to itself.");
     case NO_MODELICA_CLASS_OPEN:
@@ -597,6 +645,8 @@ QString GUIMessages::getMessage(int type)
       return tr("Unable to open file <b>%1</b>.");
     case UNABLE_TO_SAVE_FILE:
       return tr("Unable to save the file <b>%1</b>. %2");
+    case UNABLE_TO_DELETE_FILE:
+      return tr("Unable to delete <b>%1</b>.");
     case FILE_NOT_FOUND:
       return tr("The file <b>%1</b> not found.");
     case ERROR_OPENING_FILE:
@@ -626,9 +676,9 @@ QString GUIMessages::getMessage(int type)
     case INNER_MODEL_NAME_CHANGED:
       return tr("A component with the name <b>%1</b> already exists. The name is changed from <b>%1</b> to <b>%2</b>.<br /><br />This is probably wrong because the component is declared as <b>inner</b>.");
     case FMU_GENERATED:
-      return tr("The FMU %1.fmu is generated at %2");
+      return tr("The FMU is generated at <b>%1</b>.");
     case XML_GENERATED:
-      return tr("The XML is generated at %1/%2.xml");
+      return tr("The XML is generated at <b>%1</b>.");
     case FIGARO_GENERATED:
       return tr("The FIGARO is generated.");
     case UNLOAD_CLASS_MSG:
@@ -642,7 +692,7 @@ QString GUIMessages::getMessage(int type)
     case WRONG_MODIFIER:
       return tr("The Modifier <b>%1</b> format is invalid. The correct format is <b>phi(start=1)</b>");
     case SET_INFO_XML_FLAG:
-      return tr("The operations were not generated. Check Generate Operations in <b>%1->Debugger->Transformational Debugger</b> OR you must set the +d=infoXmlOperations flag via <b>%2->Simulation->OMC Flags</b> and simulate again.");
+      return tr("The operations were not generated. Check Generate Operations in <b>%1->Debugger->Transformational Debugger</b> OR you must set the -d=infoXmlOperations flag via <b>%2->Simulation->OMC Flags</b> and simulate again.");
     case DEBUG_CONFIGURATION_EXISTS_MSG:
       return tr("A debug configuration with name <b>%1</b> already exists. Error occurred while saving the debug configuration <b>%2<b>.");
     case DEBUG_CONFIGURATION_SIZE_EXCEED:
@@ -659,21 +709,29 @@ QString GUIMessages::getMessage(int type)
       return tr("The class <b>%1</b> is not a modelica class. Breakpoints are only allowed on modelica classes.");
     case TLMMANAGER_NOT_SET:
       return tr("TLM Manager executable path is not set. Set it via <b>%1->TLM</b>");
-    case METAMODEL_UNSAVED:
-      return tr("Metamodel <b>%1</b> has unsaved changes. Do you want to save?");
+    case COMPOSITEMODEL_UNSAVED:
+      return tr("CompositeModel <b>%1</b> has unsaved changes. Do you want to save?");
     case TLMCOSIMULATION_ALREADY_RUNNING:
       return tr("TLM co-simulation session is already running. Only one session is allowed.");
     case TERMINAL_COMMAND_NOT_SET:
       return tr("Terminal command is not set. You can define a new terminal command in <b>%1->General->Terminal Command</b>.");
-    case UNABLE_FIND_COMPONENT:
+    case UNABLE_FIND_COMPONENT_IN_CONNECTION:
       return tr("Unable to find component %1 while parsing connection %2.");
+    case UNABLE_FIND_COMPONENT_IN_TRANSITION:
+      return tr("Unable to find component %1 while parsing transition(%2).");
+    case UNABLE_FIND_COMPONENT_IN_INITIALSTATE:
+      return tr("Unable to find component %1 while parsing initialState(%2).");
     case SELECT_SIMULATION_OPTION:
       return tr("Select at least one of the following options, <br /><br />* %1<br />* %2<br />* %3")
           .arg(Helper::saveExperimentAnnotation)
           .arg(Helper::saveOpenModelicaSimulationFlagsAnnotation)
           .arg(Helper::simulate);
+    case INVALID_TRANSITION_CONDITION:
+      return tr("Please enter a valid condition e.g., x >=0.");
     case MULTIPLE_DECLARATIONS_COMPONENT:
       return tr("Multiple declarations of component <b>%1</b> are found.");
+    case GDB_ERROR:
+      return tr("Following error has occurred <b>%1</b> GDB arguments are <b>\"%2\"</b>");
     default:
       return "";
   }
